﻿/*****************************************************************
 * Jackson Fields
 * Collection Manager.cs
 * ***************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Engine
{
    /// <summary>
    /// Manages a large collection of objects of type T
    /// used to recycle objects to not overflow the garbage collector
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public abstract class CollectionManager<T>
    {
        protected Game game;
        public List<T> itemList = new List<T>();

        public CollectionManager(Game game)
        {
            this.game = game;
        }

        public T GetUsableItem()
        {
            T usedItem;

            foreach (T item in itemList)
            {
                if (Usable(item))
                    return item;
            }

            usedItem = NewItem();
            itemList.Add(usedItem);
            return usedItem;

        }

        //create a new item
        protected abstract T NewItem();
        protected abstract bool Usable(T item);
        public abstract void SetInactive(T item);

        public abstract void Update(GameTime gameTime);
        public abstract void Draw(SpriteBatch sb, GameTime gameTime);
    }
}
