﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Engine
{
    public class CircleCollider : CollidableObject
    {
        protected Circle circle;
        private Vector2 previousPosition;
        public Vector2 PreviousPosition
        {
            get { return previousPosition; }
            set { previousPosition = value; }
        }
        public Circle Circle
        {
            get { return circle; }
            set { circle = value; }
        }
        
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="origin"></param>
        /// <param name="radius"></param>
        public CircleCollider(Game game, AnimatedSprite sprite, Vector2 origin, float radius)
            : base(game, sprite)
        {
            Circle = new Circle(game, origin, radius);
            Move(origin);
        }

        public override bool CollidesWith(CollidableObject other)
        {
            // short circuit if we're not even collidable
            if (!IsCollidable) return false;

            if (other is CircleCollider)
            {
                return IntersectionTester.CheckIfIntersects(((CircleCollider)other).Circle, Circle);
            }
            else if (other is LineCollider)
            {
                return IntersectionTester.CheckIfIntersects(((LineCollider)other).Line, Circle);
            }

            return false;
        }
        ///
        /// <summary>
        /// Reflects a vector (vec) on a line segment
        /// </summary>
        /// <param name="vec">Vector to eb reflected</param>
        /// <param name="lp1">First point of line segment</param>
        /// <param name="lp2">Second point of line segment</param>
        /// <returns>Reflected vector</returns>
        private Vector2 Reflection(Vector2 vec, Vector2 lp1, Vector2 lp2) { 
            Vector2 normal = new Vector2(); 
            normal.X = lp2.X - lp1.X; 
            normal.Y = lp2.Y - lp1.Y; 
            float temp = normal.X; 
            normal.X = -normal.Y; 
            normal.Y = temp; 
            normal.Normalize(); 
            float dot2 = 2 * Vector2.Dot(vec, normal); 
            vec.X -= dot2 * normal.X; 
            vec.Y -= dot2 * normal.Y; 
            return vec; 
        }



        /// <summary>
        /// Bounces by default
        /// </summary>
        /// <param name="other"></param>
        public override void HandleCollision(GameTime gameTime, CollidableObject other)
        {
            if(other is LineCollider) {
                // Bounce
                //Creates segment out of velocity vector and origin
                Line tempL = new Line (circle.Origin, (circle.Origin + Velocity));

                Circle.Origin = PreviousPosition - Velocity * (float)gameTime.ElapsedGameTime.TotalSeconds; //Vector2 newPosition = sprite.Position + Velocity * (float)gameTime.ElapsedGameTime.TotalSeconds;
                base.Move(Circle.Origin);//updates sprite position.
                
                //Calculate the intersection between this new line and the lineCoolider.
                Vector2 answer = new Vector2();
                IntersectionTester.CheckIfIntersects(((LineCollider)other).Line, tempL, ref answer);
                //------------
                
                //Calculates the projection of the velocity in the lineCollider. It they collide right on the corner, then they bounce vertically.
                if (answer.X != ((LineCollider)other).Line.P1.X
                   && answer.X != ((LineCollider)other).Line.P2.X)
                {
                    Velocity = Reflection(Velocity, ((LineCollider)other).Line.P1, ((LineCollider)other).Line.P2);
                }
                else
                {
                    Velocity = -Velocity;

                }
                /*
                //hack to resolve bug
                if (PreviousPosition == circle.Origin)
                {

                    if (PreviousPosition.Y > ((LineCollider)other).Line.P1.Y
                       || PreviousPosition.Y > ((LineCollider)other).Line.P2.Y)
                    {
                        float velLength = (float)Velocity.Length();
                        Velocity = new Vector2(0, 1);
                        Velocity *= velLength;

                    }
                    else
                    {
                        float velLength = Velocity.Length();
                        Velocity = new Vector2(0, -1);
                        Velocity *= velLength;
                    }

                    Circle.Origin = PreviousPosition - Velocity * 2.0f * (float)gameTime.ElapsedGameTime.TotalSeconds; //Vector2 newPosition = sprite.Position + Velocity * (float)gameTime.ElapsedGameTime.TotalSeconds;
                    base.Move(Circle.Origin);//updates sprite position
                }
                
                */
            }
        }

        /// <summary>
        /// We have to update the origin of our circle when we move, too
        /// </summary>
        /// <param name="newLocation"></param>
        public override void Move(Vector2 newLocation)
        {
            base.Move(newLocation);
            PreviousPosition = Circle.Origin;
            Circle.Origin = newLocation;//new Vector2(newLocation.X - Circle.Radius / 2, newLocation.Y - Circle.Radius / 2);
        }

        /// <summary>
        /// Called every update loop.
        /// </summary>
        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);
        }

        /// <summary>
        /// Optionally draws the collider itself
        /// </summary>
        /// <param name="sb"></param>
        /// <param name="gameTime"></param>
        public override void Draw(Microsoft.Xna.Framework.Graphics.SpriteBatch sb, GameTime gameTime)
        {
            base.Draw(sb, gameTime);

            //Circle.Draw(game, sb, gameTime);
        }
    }
}
