﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Engine
{
    public abstract class CollidableObject : MovableObject
    {
        bool isCollidable = true;
        public bool IsCollidable
        {
            get { return isCollidable; }
            set { isCollidable = value; }
        }

        bool isRemovable = false;
        public bool IsRemovable
        {
            get { return isRemovable; }
            set { isRemovable = value; }
        }

        public CollidableObject(Game game, AnimatedSprite sprite) : base(game, sprite) {}

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);
        }

        public override void HandleOffScreen()
        {
            base.HandleOffScreen();

            // Delete objects that go off the screen
            IsRemovable = true;
        }


        /// <summary>
        /// Determines whether or not two collidable objects are intersecting
        /// </summary>
        /// <param name="other">The other object to check for collision</param>
        /// <returns>True if this object intersects the other, false otherwise</returns>
        public abstract bool CollidesWith(CollidableObject other);

        /// <summary>
        /// Called during the update loop by the CollidableObjectManager
        /// when a collision with another object is detected
        /// </summary>
        /// <param name="gameTime">Game time</param>
        /// <param name="other">The object that was collided with</param>
        public abstract void HandleCollision(GameTime gameTime, CollidableObject other);
    }
}
