﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
namespace Engine
{
    public class CollidableObjectManager
    {
        private List<CollidableObject> itemList;
        private static CollidableObjectManager instance;

        private CollidableObjectManager()
        {
            itemList = new List<CollidableObject>();
        }

        public void initialize()
        {
            itemList.RemoveRange(0, itemList.Count);
        }

        /// <summary>
        /// Singleton get instance
        /// </summary>
        /// <returns></returns>
        public static CollidableObjectManager Instance()
        {
            if (instance == null)
            {
                instance = new CollidableObjectManager();
            }
            return instance;
        }

        public void Add(CollidableObject obj)
        {
            itemList.Add(obj);
        }

        public CollidableObject ObjectAtIndex(int index)
        {
            return itemList[index];
        }

        public void Remove(int index)
        {
            itemList.RemoveAt(index);
        }

        public int getSize()
        {
            return itemList.Count;
        }

        public void Update(GameTime gameTime)
        {
            // Update the positions of all of the objects
            foreach (CollidableObject obj in itemList)
            {
                obj.Update(gameTime);
            }

            // Check for intersection of all objects
            for (int i = 0; i < itemList.Count; i++)
            {
                for (int j = i+1; j < itemList.Count; j++)
                {
                    if (itemList[i].CollidesWith(itemList[j]))
                    {
                        itemList[i].HandleCollision(gameTime, itemList[j]);
                        itemList[j].HandleCollision(gameTime, itemList[i]);
                    }
                }
            }

            // Remove any obsolete objects
            for (int i = itemList.Count - 1; i > 0; i--)
            {
                if (itemList[i].IsRemovable)
                {
                    itemList.RemoveAt(i);
                }
            }
        }

        public void Draw(SpriteBatch sb, GameTime gameTime)
        {
            // Draw all objects
            foreach (CollidableObject obj in itemList)
            {
                obj.Draw(sb, gameTime);
            }
        }
    }
}
