﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Engine
{
    static class IntersectionTester
    {


        public static double GetProjectionLength(double l1, double l2, double d)
        {
            return (Math.Pow(l1, 2) - Math.Pow(l2, 2) + Math.Pow(d, 2)) / (2 * d);
        }

        public static bool CheckIfIntersects(Circle circle, Line line)
        {
            // Circle and line
            //distance from p1 to circle origin
            Vector2 distanceP1CircleOrigin = (circle.Origin - line.P1);
            //O length of vector above and the similar vetor for p2.
            double l1 = distanceP1CircleOrigin.Length();
            double l2 = (circle.Origin - line.P2).Length();
            //distanceline.P1CircleOrigin between two LineCollider points
            double d = (line.P1 - line.P2).Length();
            //Resolves two equation system using Pitagoras triangles 
            //formed by the two vectors above and their projectiosn to the 
            //line connecting the two line points
            double proj1 = GetProjectionLength( l1, l2, d);
            //Calcaultates projection vector for distanceline.P1CircleOrigin on line 
            //connecting the two poitns fo the line.
            Vector2 tempV = (line.P2 - line.P1);
            tempV.Normalize();
            Vector2 proj1V = tempV * ((float)proj1);
            //If the the sum of the distances between the two line points and 
            //proj1 is greater that the distance of the two points, the circle 
            //is outside of the line segment. hence, collision should be dealt 
            //differently.
            float distProj1P2 = ((line.P2 - (line.P1 + proj1V)).Length());
            float distProj1P1 = proj1V.Length();
            if ((distProj1P1 + distProj1P2) > d)
            {
                if (l1 > circle.Radius && l2 > circle.Radius)
                    return false;
                else
                    return true;
            }
            //Gets the distance between distanceline.P1CircleOrigin and its projection (proj1V)
            Vector2 distanceToProjection1 = distanceP1CircleOrigin - proj1V;

            return circle.Radius >= distanceToProjection1.Length();
        }
        public static bool CheckIfIntersects(Line line, Circle circle)
        {
            return CheckIfIntersects(circle, line);
        }
        public static bool CheckIfIntersects(Circle circle1, Circle circle2)
        {
            // Two circles
            float distance = (circle1.Origin - circle2.Origin).Length();
            float sum = circle1.Radius + circle2.Radius;
            return distance <= sum;
        }
        public static bool CheckIfIntersects(Line line1, Line line2)
        {
            Vector2 answer = new Vector2(0,0);
            return CheckIfIntersects(line1, line2, ref answer);
        }
        public static bool CheckIfIntersects(Line line1, Line line2, ref Vector2 answer)
        {
            // Two lines
            // Store the values for fast access and easy
            // equations-to-code conversion
            float x1 = line1.P1.X, x2 = line1.P2.X, x3 = line2.P1.X, x4 = line2.P2.X;
            float y1 = line1.P1.Y, y2 = line1.P2.Y, y3 = line2.P1.Y, y4 = line2.P2.Y;

            float d = (x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4);
            // If d is zero, there is no intersection
            if (d == 0) return false;

            // Get the x and y
            float pre = (x1 * y2 - y1 * x2), post = (x3 * y4 - y3 * x4);
            float x = (pre * (x3 - x4) - (x1 - x2) * post) / d;
            float y = (pre * (y3 - y4) - (y1 - y2) * post) / d;

            // Check if the x and y coordinates are within both lines
            if (x < Math.Min(x1, x2) || x > Math.Max(x1, x2) ||
                x < Math.Min(x3, x4) || x > Math.Max(x3, x4)) return false;
            if (y < Math.Min(y1, y2) || y > Math.Max(y1, y2) ||
                y < Math.Min(y3, y4) || y > Math.Max(y3, y4)) return false;

            // Return the point of intersection
            answer.X = x;
            answer.Y = y;

            return true;
        }

    }
}
