﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Engine
{
    public class LineCollider : CollidableObject
    {
        Line line;
        public Line Line
        {
            get { return line; }
            set { line = value; }
        }
        
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="origin"></param>
        /// <param name="radius"></param>
        public LineCollider(Game game, AnimatedSprite sprite, Vector2 p1, Vector2 p2)
            : base(game, sprite)
        {
            Line = new Line(p1, p2);
        }

        public override bool CollidesWith(CollidableObject other)
        {
            // short circuit if we're not even collidable
            if (!IsCollidable) return false;

            if (other is CircleCollider)
            {
                return IntersectionTester.CheckIfIntersects(((CircleCollider)other).Circle, Line);
            }
            else if (other is LineCollider)
            {
                return IntersectionTester.CheckIfIntersects(((LineCollider)other).Line, Line);
            }

            return false;
        }

        public override void HandleCollision(GameTime gameTime, CollidableObject other)
        {
            // Doesn't do anything
        }

        /// <summary>
        /// We have to update the origin of our line when we move, too
        /// </summary>
        /// <param name="newLocation"></param>
        public override void Move(Vector2 newLocation)
        {
            base.Move(newLocation);
            Vector2 tempV = (Line.P2 - Line.P1);
            float halfLength = tempV.Length() * 0.5f;
            tempV.Normalize();
            Line.P1 = newLocation - (tempV * halfLength);
            Line.P2 = newLocation + (tempV * halfLength);
        }

        public override void Draw(SpriteBatch sb, GameTime gameTime)
        {
            base.Draw(sb, gameTime);
        }
    }
}
