﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Engine;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Engine
{
    public abstract class MovableObject
    {
        protected Game game;

        private float rotationAngle;
        public float RotationAngle
        {
            get { return rotationAngle; }
            set { rotationAngle = value; }
        }
        private float rotationAngleSpeed;

        public float RotationAngleSpeed
        {
            get { return rotationAngleSpeed; }
            set { rotationAngleSpeed = value; }
        }

        AnimatedSprite sprite;
        public AnimatedSprite Sprite
        {
            get { return sprite; }
            set { sprite = value; }
        }

        Vector2 velocity;
        public Vector2 Velocity
        {
            get { return velocity; }
            set { velocity = value; }
        }
        bool gravityEnabled;

        public bool GravityEnabled
        {
            get { return gravityEnabled; }
            set { gravityEnabled = value; }
        }

        private static float gravity = 25.0f;
        public MovableObject(Game game, AnimatedSprite sprite)
        {
            this.game = game;
            Sprite = sprite;
            Velocity = new Vector2(0, 0);
            GravityEnabled = false;
            
        }
        public MovableObject(Game game, AnimatedSprite sprite, Vector2 velocity)
        {
            
            Sprite = sprite;
            Velocity = velocity;
            RotationAngle = 0.0f;
            GravityEnabled = false;
        }

        /// <summary>
        /// Called every update loop
        /// </summary>
        public virtual void Update(GameTime gameTime)
        {
            sprite.Update(gameTime);
            //PreviousPosition = sprite.Position;
            Vector2 newPosition = sprite.Position + Velocity * (float)gameTime.ElapsedGameTime.TotalSeconds;
            if (gravityEnabled)
                Velocity = Velocity  +  new Vector2(0,1) * gravity * (float)gameTime.ElapsedGameTime.TotalSeconds;

            Move(newPosition);

            RotationAngle += RotationAngleSpeed * (float)gameTime.ElapsedGameTime.TotalSeconds;

            // Notify objects when they go off screen
            Viewport vp = game.GraphicsDevice.Viewport;
            //if(!sprite.Bounds().Intersects(game.GraphicsDevice.Viewport.Bounds))
            float halfWidth = sprite.Bounds().Width * 0.5f;
            float halfHeight = sprite.Bounds().Height * 0.5f;
            if (sprite.Position.X < 0 - halfWidth || sprite.Position.Y < 0 - halfHeight || sprite.Position.X - halfWidth > vp.Width || sprite.Position.Y - halfHeight > vp.Height)
            {
                HandleOffScreen();
            }
        }

        /// <summary>
        /// By default, this function does nothing, but it is called every update that the object is off screen
        /// </summary>
        public virtual void HandleOffScreen()
        {

        }

        /// <summary>
        /// Changes the location of this movable object
        /// </summary>
        /// <param name="newLocation">The location to jump this object to</param>
        public virtual void Move(Vector2 newLocation)
        {
            sprite.Position = newLocation;
        }

        /// <summary>
        /// Draw the sprite and stuff
        /// </summary>
        /// <param name="gameTime"></param>
        public virtual void Draw(SpriteBatch sb, GameTime gameTime)
        {
            //sprite.Draw(sb, gameTime);
            sprite.Draw(sb, gameTime, RotationAngle);
        }
    }
}
