﻿/*****************************************************************
 * Jackson Fields
 * AnimatedSprite.cs
 * ***************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Engine
{
    /// <summary>
    /// manages animations
    /// </summary>
    public class AnimatedSprite : Sprite
    {
        protected Boolean Paused;
        protected Boolean PlayLooped;
        protected float TotalElapsed;

        protected float frameWidth;
        protected float frameTime;

        protected int FrameNum;
        //Inidcates the idnex of the frame the sprite should be kept paused in.
        protected int pauseFrame;

        public AnimatedSprite(Game game, Texture2D texture, Vector2 position, float frameWidth, float frameTime, Boolean PlayLooped, Boolean StartPaused, int pauseFrame)
            : base(game, texture, position)
        {
            this.frameTime = frameTime;
            this.frameWidth = frameWidth;
            this.pauseFrame = (pauseFrame>=0)?pauseFrame:0;
            this.PlayLooped = PlayLooped;
            this.Paused = StartPaused;

            FrameNum = 0;

            bounds = new Rectangle(bounds.X, bounds.Y, (int)frameWidth, (int)texture.Height);
        }

        public void PlayAnimation()
        {
            FrameNum = 0;
            Paused = false;
        }

        public Boolean IsPaused()
        {
            return Paused;
        }

        public void SetLooped(Boolean isLooped)
        {
            PlayLooped = isLooped;
        }

        public override void Update(GameTime gameTime)
        {
            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;
            UpdateFrame(elapsed);

            base.Update(gameTime);
        }

        public void UpdateFrame(float elapsed)
        {
            if (Paused)
                return;
            TotalElapsed += elapsed;
            if (TotalElapsed > frameTime)
            {
                FrameNum++;

                // Keep the Frame between 0 and the total frames, minus one.
                if (FrameNum >= Texture.Width / (int)frameWidth)
                {
                    FrameNum = pauseFrame;
                    //FrameNum = 0;

                    if (!PlayLooped)
                        Paused = true;
                }
                TotalElapsed -= frameTime;
            }
        }


        public override void Draw(SpriteBatch sb, GameTime gameTime)
        {
                sb.Draw(Texture, Position, new Rectangle((int)(FrameNum * frameWidth), 0, (int)frameWidth, Texture.Height), Color.White);
        }

        public void Draw(SpriteBatch sb, GameTime gameTime, Vector2 offset, float scale, float alpha)
        {
                sb.Draw(Texture, Position + new Vector2(frameWidth / 2, Texture.Height / 2) + offset, new Rectangle((int)(FrameNum * frameWidth), 0, (int)frameWidth, Texture.Height), Color.White * alpha, 0, new Vector2(frameWidth / 2, Texture.Height / 2), scale, SpriteEffects.None, 1);
        }

        public void Draw(SpriteBatch sb, GameTime gameTime, Vector2 offset, float angle, float scale, float alpha)
        {
            sb.Draw(Texture, Position + new Vector2(frameWidth / 2, Texture.Height / 2) + offset, new Rectangle((int)(FrameNum * frameWidth), 0, (int)frameWidth, Texture.Height), Color.White * alpha, angle, new Vector2(frameWidth / 2, Texture.Height / 2), scale, SpriteEffects.None, 1);
        }

        public void Draw(SpriteBatch sb, GameTime gameTime, float angle)
        {
                sb.Draw(Texture, Position, new Rectangle((int)(FrameNum * frameWidth), 0, (int)frameWidth, Texture.Height), Color.White, angle, new Vector2(Texture.Width * 0.5f, Texture.Height * 0.5f), 1.0f, SpriteEffects.None, 1);
        }

        public void Draw(SpriteBatch sb, GameTime gameTime, float angle, float alpha)
        {
            sb.Draw(Texture, Position, new Rectangle((int)(FrameNum * frameWidth), 0, (int)frameWidth, Texture.Height), Color.White * alpha, angle, new Vector2(Texture.Width * 0.5f, Texture.Height * 0.5f), 1.0f, SpriteEffects.None, 1);
        }

        public override void Draw(SpriteBatch sb, GameTime gameTime, SpriteEffects effects)
        {
                sb.Draw(Texture, Position, new Rectangle((int)(FrameNum * frameWidth), 0, (int)frameWidth, Texture.Height),
                Color.White, 0, Vector2.Zero, Vector2.One, effects, 0);
        }
    }
}
