﻿/*****************************************************************
 * Jackson Fields
 * Camera.cs
 * ***************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
//using Microsoft.Xna.Framework.Net;
//using Microsoft.Xna.Framework.Storage;

namespace Engine
{
    public class Camera
    {
        Game game;
        public Vector2 Position;
        int CameraMoveOffset = 3; //offset from bounds to move camera
        Rectangle cameraBounds;

        public Camera(Game game, Vector2 startPos, Rectangle cameraBounds)
        {
            this.game = game;
            SetBounds(startPos - new Vector2(game.GraphicsDevice.Viewport.Width / 2, game.GraphicsDevice.Viewport.Height / 2), 
                cameraBounds);
        }

        public void SetBounds(Vector2 startPos, Rectangle cameraBounds)
        {
            this.cameraBounds = cameraBounds;
            Vector2 camPos = startPos;

            Position = new Vector2(MathHelper.Clamp(camPos.X, cameraBounds.X, cameraBounds.X + cameraBounds.Width), 
                MathHelper.Clamp(camPos.Y, cameraBounds.Y, cameraBounds.Y + cameraBounds.Height));
        }

        public void Update(Sprite following)
        {
            //move off screen right
            if (following.Position.X + (following.Texture.Width / 2) > Position.X + ((CameraMoveOffset - 1) * game.GraphicsDevice.Viewport.Width / CameraMoveOffset))
            {
                Vector2 camPos = following.Position + new Vector2(following.Texture.Width / 2, 0) - new Vector2((CameraMoveOffset - 1) * game.GraphicsDevice.Viewport.Width / CameraMoveOffset, 0);
                if (camPos.X < cameraBounds.X + cameraBounds.Width - game.GraphicsDevice.Viewport.Width)
                    Position.X = camPos.X;
            }
            //move off screen bottom
            if (following.Position.Y + (following.Texture.Height / 2) > Position.Y + ((CameraMoveOffset - 1) * game.GraphicsDevice.Viewport.Height / CameraMoveOffset))
            {
                Vector2 camPos = following.Position + new Vector2(0, following.Texture.Height / 2) - new Vector2(0, (CameraMoveOffset - 1) * game.GraphicsDevice.Viewport.Height / CameraMoveOffset);
                if (camPos.Y < cameraBounds.Y + cameraBounds.Height - game.GraphicsDevice.Viewport.Height)
                    Position.Y = camPos.Y;
            }

            //move off screen left
            if (following.Position.X + (following.Texture.Width / 2) < Position.X + (game.GraphicsDevice.Viewport.Width / CameraMoveOffset))
            {
                Vector2 camPos = following.Position + new Vector2(following.Texture.Width / 2, 0) - new Vector2(game.GraphicsDevice.Viewport.Width / CameraMoveOffset, 0);
                if (camPos.X > cameraBounds.X)
                    Position.X = camPos.X;
            }
            //move off screen top
            if (following.Position.Y + (following.Texture.Height / 2) < Position.Y + (game.GraphicsDevice.Viewport.Height / CameraMoveOffset))
            {
                Vector2 camPos = following.Position + new Vector2(0, following.Texture.Height / 2) - new Vector2(0, game.GraphicsDevice.Viewport.Height / CameraMoveOffset);
                if (camPos.Y > cameraBounds.Y)
                    Position.Y = camPos.Y;
            }
        }

        public void Translate(Vector2 moveVector)
        {
            Position += moveVector;
        }

        public Matrix Transformation()
        {
            return Matrix.CreateTranslation(new Vector3(-Position, 0));
        }

        public Matrix InverseTransformation()
        {
            return Matrix.Invert(Transformation());
        }

        public void Draw(SpriteBatch sb, GameTime gameTime)
        {
            sb.End(); //camera uses it's own SB
            sb.Begin(SpriteSortMode.Immediate, null, null, null, null, null, Transformation());
        }
    }
}
