﻿/*****************************************************************
 * Jackson Fields
 * Sprite.cs
 * ***************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Engine
{
    /// <summary>
    /// holds texture and position
    /// </summary>
    public class Sprite
    {
        protected Rectangle bounds;
        protected Game game;
        public virtual Vector2 Position { get; set; }
        public Texture2D Texture { get; set; }

        public Sprite(Game game, Texture2D texture, Vector2 position)
        {
            this.game = game;
            this.Texture = texture;
            this.Position = position;

            bounds = Texture.Bounds;
        }

        public virtual void Update(GameTime gameTime)
        {
        }

        public Rectangle Bounds()
        {
            return new Rectangle((int)(Position.X + bounds.X), (int)(Position.Y + bounds.Y), bounds.Width, bounds.Height);
        }

        public virtual Boolean Contains(Vector2 position)
        {
            return Bounds().Contains((int)position.X, (int)position.Y);
        }

        public virtual void Draw(SpriteBatch sb, GameTime gameTime)
        {
            sb.Draw(Texture, Position, Color.White);
        }

        public virtual void Draw(SpriteBatch sb, GameTime gameTime, SpriteEffects effects)
        {
            sb.Draw(Texture, Position, null, Color.White, 0, Vector2.Zero, Vector2.One, effects, 0);
        }
    }
}

