﻿/*****************************************************************
 * Jackson Fields
 * EngineGame.cs
 * ***************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Engine
{
    /// <summary>
    /// some fields for a basic game framework
    /// </summary>
    public class EngineGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        //generic font
        public SpriteFont font;

        //Components
        public Input input;

        //Layouts
        public Layout currentLayout;

        public Boolean isMuted = false;

        public EngineGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            //Components
            input = new Input(this);
            Components.Add(input);

            base.Initialize();
        }

        protected void SetResolution(int width, int height)
        {
            graphics.PreferredBackBufferWidth = width;
            graphics.PreferredBackBufferHeight = height;
            graphics.ApplyChanges();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            font = Content.Load<SpriteFont>("Font/font");
        }

        protected override void UnloadContent()
        {
        }

        protected override void Update(GameTime gameTime)
        {
            currentLayout.Update(gameTime);

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            //Pre Draw
            GraphicsDevice.Clear(Color.Black);
            spriteBatch.Begin();

            currentLayout.Draw(spriteBatch, gameTime);
            Draw(spriteBatch, gameTime);

            //Post Draw
            spriteBatch.End();
            base.Draw(gameTime);
        }

        protected virtual void Draw(SpriteBatch sb, GameTime gameTime) { }
    }
}
