﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Kinect;

namespace Engine
{
    public class Kinect
    {
        KinectSensor kinectSensor;
        DepthImageFrame depthFrame;
        ColorImageFrame colorFrame;
        private byte[] pixelData;

        Texture2D cameraTexture;

        Game game;

        int sensor = 0;

        public Kinect(Game game, int sensor = 0)
        {
            this.game = game;
            this.sensor = sensor;

            Initialize();
        }

        public void ToggleDistance()
        {
            if (isKinectOn())
            {
                if (kinectSensor.DepthStream.Range == DepthRange.Near)
                {
                    kinectSensor.DepthStream.Range = DepthRange.Default;
                }
                else
                {
                    kinectSensor.DepthStream.Range = DepthRange.Near;
                }
            }
        }

        public void Initialize()
        {
            try
            {
                kinectSensor = KinectSensor.KinectSensors[sensor];

                kinectSensor.DepthStream.Enable(DepthImageFormat.Resolution320x240Fps30);
                kinectSensor.DepthStream.Range = DepthRange.Near;
                kinectSensor.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30);

                StartSensor();
            }
            catch { }
        }

        public void StartSensor()
        {
            if (kinectSensor == null)
            {
                Initialize();
            }
            try
            {
                kinectSensor.Start();
            }
            catch { }
        }

        public void StopSensor()
        {
            if (isKinectOn())
            {
                kinectSensor.Stop();
            }
        }

        public void UnloadContent()
        {
            try
            {
                kinectSensor.Stop();
            }
            catch { }
        }

        int ELEVATION_CHANGE = 5;
        public void RotateDown()
        {
            try
            {
                if (isKinectOn())
                {
                    if (kinectSensor.ElevationAngle - ELEVATION_CHANGE >= kinectSensor.MinElevationAngle)
                    {
                        kinectSensor.ElevationAngle -= ELEVATION_CHANGE;
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        public void RotateUp()
        {
            try
            {
                if (isKinectOn())
                {
                    if (kinectSensor.ElevationAngle + ELEVATION_CHANGE <= kinectSensor.MaxElevationAngle)
                    {
                        kinectSensor.ElevationAngle += ELEVATION_CHANGE;
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        List<Vector3> cloud = new List<Vector3>();
        public PointCloud GetPointCloud(int mod = 1, float minX = float.MinValue, float maxX = float.MaxValue, float minY = float.MinValue, float maxY = float.MaxValue, float minZ = 0, float maxZ = float.MaxValue)
        {
            if (isKinectOn())
            {
                if (depthFrame != null)
                {
                    depthFrame.Dispose();
                }
                depthFrame = kinectSensor.DepthStream.OpenNextFrame(0);

                if (depthFrame != null)
                {
                    cloud.Clear();
                    short[] pixelData = new short[depthFrame.PixelDataLength];

                    depthFrame.CopyPixelDataTo(pixelData);

                    for (int x = 0; x < depthFrame.Width; x += mod)
                    {
                        for (int y = 0; y < depthFrame.Height; y += mod)
                        {
                            SkeletonPoint skelPoint = kinectSensor.MapDepthToSkeletonPoint(DepthImageFormat.Resolution320x240Fps30, x, y, pixelData[x + y * depthFrame.Width]);
                            if (skelPoint.X > minX && skelPoint.X < maxX &&
                                skelPoint.Y > minY && skelPoint.Y < maxY &&
                                skelPoint.Z > minZ && skelPoint.Z < maxZ)
                            {
                                cloud.Add(new Vector3(skelPoint.X, skelPoint.Y, skelPoint.Z));
                            }
                        }
                    }
                }
            }

            return new PointCloud(game, cloud);
        }

        public Texture2D getCameraTexture()
        {
            if (isKinectOn())
            {
                if (colorFrame != null)
                {
                    colorFrame.Dispose();
                }
                colorFrame = kinectSensor.ColorStream.OpenNextFrame(0);

                if (colorFrame != null)
                {
                    this.pixelData = new byte[colorFrame.PixelDataLength];
                    cameraTexture = new Texture2D(game.GraphicsDevice, colorFrame.Width, colorFrame.Height);

                    colorFrame.CopyPixelDataTo(this.pixelData);
                    cameraTexture.SetData<byte>(pixelData);
                }
            }

            return cameraTexture;
        }

        public Boolean isKinectOn()
        {
            if (kinectSensor == null)
            {
                return false;
            }

            return kinectSensor != null && kinectSensor.IsRunning;
        }
    }
}
