﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Engine
{
    public class PointCloud
    {
        public List<Vector3> points;
        List<Triangle> triangles = new List<Triangle>();
        List<VertexPositionColor> vertices = new List<VertexPositionColor>();
        List<int> indices = new List<int>();
        VertexBuffer myVertexBuffer;
        IndexBuffer myIndexBuffer;

        Effect effect;
        Game game;

        public PointCloud(Game game, List<Vector3> points, int mod = 1)
        {
            this.game = game;
            this.effect = game.Content.Load<Effect>("SimpleEffect");
            Create(points, mod);
        }

        public void Create(List<Vector3> points, int mod = 1)
        {
            this.points = points;

            foreach (Vector3 point in points)
            {
                triangles.Add(new Triangle(game, point, Vector3.One));
            }

            for (int i = 0, y = 0; i < points.Count; i++)
            {
                if (i % mod == 0)
                {
                    vertices.AddRange(triangles[i].verts);
                    indices.Add(y);
                    indices.Add(y + 1);
                    indices.Add(y + 2);

                    y += 3;
                }
            }

            CopyToBuffers();
        }

        private void CopyToBuffers()
        {
            if (vertices.Count > 0)
            {
                myVertexBuffer = new VertexBuffer(game.GraphicsDevice, VertexPositionColor.VertexDeclaration, vertices.ToArray().Length, BufferUsage.WriteOnly);
                myVertexBuffer.SetData(vertices.ToArray());

                myIndexBuffer = new IndexBuffer(game.GraphicsDevice, typeof(int), indices.ToArray().Length, BufferUsage.WriteOnly);
                myIndexBuffer.SetData(indices.ToArray());
            }
        }

        public void Draw(FPSCamera camera)
        {
            effect.Parameters["World"].SetValue(Matrix.Identity);
            effect.Parameters["View"].SetValue(camera.View);
            effect.Parameters["Projection"].SetValue(camera.Projection);

            if (vertices.Count > 0)
            {
                foreach (EffectPass pass in effect.Techniques[0].Passes)
                {
                    pass.Apply();

                    game.GraphicsDevice.Indices = myIndexBuffer;
                    game.GraphicsDevice.SetVertexBuffer(myVertexBuffer);
                    game.GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.TriangleList,
                        0, 0, vertices.ToArray().Length, 0, indices.ToArray().Length / 3);
                }
            }
        }
    }
}
