﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Engine
{
    public static class ModelMethods
    {
        public static Vector3 GetMeshPosition(Model model, Matrix[] transforms, Matrix World, ModelMesh mesh)
        {
            model.CopyAbsoluteBoneTransformsTo(transforms);

            Matrix meshWorld = (transforms[mesh.ParentBone.Index] * World);

            Vector3 scale;
            Quaternion rotation;
            Vector3 position;

            meshWorld.Decompose(out scale, out rotation, out position);

            return position;
        }

        public static void DrawBasicModel(Matrix Projection, Matrix View, Model model, Matrix World, Matrix[] transforms)//, List<String> meshExclusionList)
        {
            //model.CopyAbsoluteBoneTransformsTo(transforms);

            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.EnableDefaultLighting();
                    //effect.LightingEnabled = false;
                    effect.PreferPerPixelLighting = true;

                    effect.Projection = Projection;
                    effect.View = View;

                    effect.World = transforms[mesh.ParentBone.Index] * World;
                }

                //TODO contains
                //if (meshExclusionList == null || !meshExclusionList.Contains(mesh.Name))
                //{
                if (!mesh.Name.ToUpper().Contains("BOUND"))
                {
                    mesh.Draw();
                }
            }
        }

        //public static void DrawBasicModel(Matrix Projection, Matrix View, Model model, Matrix World)
        //{
        //    foreach (ModelMesh mesh in model.Meshes)
        //    {
        //        foreach (BasicEffect effect in mesh.Effects)
        //        {
        //            effect.EnableDefaultLighting();
        //            effect.PreferPerPixelLighting = true;

        //            effect.Projection = Projection;
        //            effect.View = View;
        //            effect.World = World;
        //        }

        //        mesh.Draw();
        //    }
        //}

        //public static void DrawSkinnedModel(AnimationController animationController, Matrix Projection, Matrix View, Model Model, Matrix World, Matrix[] transforms)
        //{
        //    //Model.CopyAbsoluteBoneTransformsTo(transforms);

        //    foreach (ModelMesh modelMesh in Model.Meshes)
        //    {
        //        foreach (SkinnedModelBasicEffect effect in modelMesh.Effects)
        //        {
        //            // Setup world transform - why aren't the transforms working?
        //            effect.World = /*transforms[modelMesh.ParentBone.Index] */ World;

        //            // Setup camera
        //            effect.View = View;
        //            effect.Projection = Projection;

        //            // Set the animated bones to the model
        //            effect.Bones = animationController.SkinnedBoneTransforms;

        //            // OPTIONAL - Configure material
        //            effect.Material.DiffuseColor = new Vector3(0.8f);
        //            effect.Material.SpecularColor = new Vector3(0.3f);
        //            effect.Material.SpecularPower = 8;

        //            // OPTIONAL - Configure lights
        //            effect.AmbientLightColor = new Vector3(0.1f);
        //            effect.LightEnabled = true;
        //            effect.EnabledLights = EnabledLights.One;
        //            effect.PointLights[0].Color = Vector3.One;
        //            effect.PointLights[0].Position = new Vector3(100);
        //        }

        //        // Draw a model mesh
        //        modelMesh.Draw();
        //    }
        //}

        ////public static void DrawSkinnedModel(AnimationController animationController, Matrix Projection, Matrix View, SkinnedModel SkinnedModel, Matrix World)
        ////{
        ////}

        //public static void ResetRenderState(GraphicsDevice graphics)
        //{
        //    graphics.RenderState.DepthBufferEnable = true;
        //    graphics.RenderState.AlphaBlendEnable = false;
        //    graphics.RenderState.AlphaTestEnable = false;
        //    graphics.SamplerStates[0].AddressU = TextureAddressMode.Wrap;
        //    graphics.SamplerStates[0].AddressV = TextureAddressMode.Wrap;
        //    graphics.RenderState.CullMode = CullMode.CullCounterClockwiseFace;
        //}

        public struct TriangleVertexIndices
        {
            public int I0;
            public int I1;
            public int I2;
        }

        public static void ExtractData(Model mdl, Matrix World, List<Vector3> vtcs, List<TriangleVertexIndices> idcs, bool includeNoncoll)
        {
            Matrix m = Matrix.Identity;
            foreach (ModelMesh mm in mdl.Meshes)
            {
                m = World; // GetAbsoluteTransform(mm.ParentBone);
                ExtractModelMeshData(mm, ref m, vtcs, idcs, includeNoncoll);
            }
        }

        public static Matrix GetAbsoluteTransform(ModelBone bone)
        {
            if (bone == null)
                return Matrix.Identity;
            return bone.Transform * GetAbsoluteTransform(bone.Parent);
        }

        public static void ExtractModelMeshData(ModelMesh mm, ref Matrix xform,
            List<Vector3> vertices, List<TriangleVertexIndices> indices, bool includeNoncoll = true)
        {
            foreach (ModelMeshPart mmp in mm.MeshParts)
            {

                if (!includeNoncoll)
                {
                    EffectAnnotation annot = mmp.Effect.CurrentTechnique.Annotations["collide"];
                    if (annot != null && annot.GetValueBoolean() == false)
                    {
                        Console.WriteLine("Ignoring model mesh part {0}:{1} because it's set to not collide.",
                            mm.Name);
                        continue;
                    }
                }
                ExtractModelMeshPartData(mmp, ref xform, vertices, indices);
            }
        }

        /// <summary>  
        /// Get all the triangles from each mesh part (Changed for XNA 4)  
        /// </summary>  
        public static void ExtractModelMeshPartData(ModelMeshPart meshPart, ref Matrix transform,
            List<Vector3> vertices, List<TriangleVertexIndices> indices)
        {
            // Before we add any more where are we starting from  
            int offset = vertices.Count;

            // == Vertices (Changed for XNA 4.0)  

            // Read the format of the vertex buffer  
            VertexDeclaration declaration = meshPart.VertexBuffer.VertexDeclaration;
            VertexElement[] vertexElements = declaration.GetVertexElements();
            // Find the element that holds the position  
            VertexElement vertexPosition = new VertexElement();
            foreach (VertexElement vert in vertexElements)
            {
                if (vert.VertexElementUsage == VertexElementUsage.Position &&
                    vert.VertexElementFormat == VertexElementFormat.Vector3)
                {
                    vertexPosition = vert;
                    // There should only be one  
                    break;
                }
            }
            // Check the position element found is valid  
            if (vertexPosition == null ||
                vertexPosition.VertexElementUsage != VertexElementUsage.Position ||
                vertexPosition.VertexElementFormat != VertexElementFormat.Vector3)
            {
                throw new Exception("Model uses unsupported vertex format!");
            }
            // This where we store the vertices until transformed  
            Vector3[] allVertex = new Vector3[meshPart.NumVertices];
            // Read the vertices from the buffer in to the array  
            meshPart.VertexBuffer.GetData<Vector3>(
                meshPart.VertexOffset * declaration.VertexStride + vertexPosition.Offset,
                allVertex,
                0,
                meshPart.NumVertices,
                declaration.VertexStride);
            // Transform them based on the relative bone location and the world if provided  
            for (int i = 0; i != allVertex.Length; ++i)
            {
                Vector3.Transform(ref allVertex[i], ref transform, out allVertex[i]);
            }
            // Store the transformed vertices with those from all the other meshes in this model  
            vertices.AddRange(allVertex);

            // == Indices (Changed for XNA 4)  

            // Find out which vertices make up which triangles  
            if (meshPart.IndexBuffer.IndexElementSize != IndexElementSize.SixteenBits)
            {
                // This could probably be handled by using int in place of short but is unnecessary  
                throw new Exception("Model uses 32-bit indices, which are not supported.");
            }
            // Each primitive is a triangle  
            short[] indexElements = new short[meshPart.PrimitiveCount * 3];
            meshPart.IndexBuffer.GetData<short>(
                meshPart.StartIndex * 2,
                indexElements,
                0,
                meshPart.PrimitiveCount * 3);
            // Each TriangleVertexIndices holds the three indexes to each vertex that makes up a triangle  
            TriangleVertexIndices[] tvi = new TriangleVertexIndices[meshPart.PrimitiveCount];
            for (int i = 0; i != tvi.Length; ++i)
            {
                // The offset is because we are storing them all in the one array and the   
                // vertices were added to the end of the array.  
                tvi[i].I0 = indexElements[i * 3 + 0] + offset;
                tvi[i].I1 = indexElements[i * 3 + 1] + offset;
                tvi[i].I2 = indexElements[i * 3 + 2] + offset;
            }
            // Store our triangles  
            indices.AddRange(tvi);
        }  
    }
}
