﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Engine
{
    public class LineManager
    {
        Effect effect;

        List<VertexPositionColor> linepoints = new List<VertexPositionColor>();
        List<short> indices = new List<short>();

        VertexBuffer myVertexBuffer;
        IndexBuffer myIndexBuffer;

        Game game;

        public LineManager(Game game)
        {
            this.game = game;
            effect = game.Content.Load<Effect>("SimpleEffect");
        }

        public void Draw(FPSCamera camera)
        {
            if (linepoints.Count > 0)
            {
                DrawLines(camera);
            }
        }

        public void ClearLines()
        {
            linepoints.Clear();
            indices.Clear();
            indexCounter = 0;
        }

        int indexCounter = 0;
        public void AddLine(Line line)
        {
            linepoints.Add(new VertexPositionColor(line.start, Color.Red));
            linepoints.Add(new VertexPositionColor(line.end, Color.Red));
            for (int i = 0; i < 6; i++)
            {
                indices.Add((short)indexCounter);
                indexCounter++;
            }

            CopyToBuffers();
        }

        public void AddLine(Line line, Color color)
        {
            linepoints.Add(new VertexPositionColor(line.start, color));
            linepoints.Add(new VertexPositionColor(line.end, color));
            for (int i = 0; i < 6; i++)
            {
                indices.Add((short)indexCounter);
                indexCounter++;
            }

            CopyToBuffers();
        }

        private void CopyToBuffers()
        {
            myVertexBuffer = new VertexBuffer(game.GraphicsDevice, VertexPositionColor.VertexDeclaration, linepoints.Count, BufferUsage.WriteOnly);
            myVertexBuffer.SetData(linepoints.ToArray());

            myIndexBuffer = new IndexBuffer(game.GraphicsDevice, typeof(short), indices.Count, BufferUsage.WriteOnly);
            myIndexBuffer.SetData(indices.ToArray());
        }

        private void DrawLines(FPSCamera camera)
        {
            effect.Parameters["World"].SetValue(Matrix.Identity);
            effect.Parameters["View"].SetValue(camera.View);
            effect.Parameters["Projection"].SetValue(camera.Projection);

            foreach (EffectPass pass in effect.Techniques[0].Passes)
            {
                pass.Apply();

                game.GraphicsDevice.Indices = myIndexBuffer;
                game.GraphicsDevice.SetVertexBuffer(myVertexBuffer);
                game.GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.LineList, 0, 0, linepoints.Count, 0, linepoints.Count);
            }
        }
    }
}
