﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Engine
{
    public class Triangle
    {
        Game game;
        public Vector3 position;
        Vector3 color;

        float triDist = 0.005f;

        public VertexPositionColor[] verts;

        /// <summary>
        /// use this to create a triangle for a point
        /// the tri vertices will be created from the location
        /// </summary>
        public Triangle(Game game, Vector3 position, Vector3 color)
        {
            this.game = game;
            this.position = position;
            this.color = color;

            Color col = new Color(color.X, color.Y, color.Z, 1);
            verts = new VertexPositionColor[3];

            verts[0].Position = position;
            verts[0].Color = col;

            verts[1].Position = position + new Vector3(triDist, triDist, 0);
            verts[1].Color = col;

            verts[2].Position = position + new Vector3(-triDist, triDist, 0);
            verts[2].Color = col;

            RasterizerState rasterizerState = new RasterizerState();
            rasterizerState.CullMode = CullMode.None;
            game.GraphicsDevice.RasterizerState = rasterizerState;
        }

        /// <summary>
        /// use this to create a trangle with defined vertices
        /// </summary>
        public Triangle(Game game, Vector3 position, Vector3 point1, Vector3 point2, Vector3 point3, Vector3 color)
        {
            this.game = game;
            this.position = position;
            this.color = color;

            Color col = new Color(color.X, color.Y, color.Z, 1);
            verts = new VertexPositionColor[3];

            verts[0].Position = point1;
            verts[0].Color = col;

            verts[1].Position = point2;
            verts[1].Color = col;

            verts[2].Position = point3;
            verts[2].Color = col;

            RasterizerState rasterizerState = new RasterizerState();
            rasterizerState.CullMode = CullMode.None;
            game.GraphicsDevice.RasterizerState = rasterizerState;
        }

        public void Draw(FPSCamera camera, Effect effect)
        {
            effect.Parameters["World"].SetValue(Matrix.CreateTranslation(position));
            effect.Parameters["View"].SetValue(camera.View);
            effect.Parameters["Projection"].SetValue(camera.Projection);
            //effect.Parameters["Color"].SetValue(new Vector4(color.X, color.Y, color.Z, 1));

            foreach (EffectPass pass in effect.Techniques[0].Passes)
            {
                pass.Apply();

                game.GraphicsDevice.DrawUserPrimitives(
                    PrimitiveType.TriangleList, verts,
                    0, 1);//, VertexPositionColor.VertexDeclaration);
            }
        }
    }
}
