﻿/*****************************************************************
 * Jackson Fields
 * UIButton.cs
 * ***************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Engine
{
    /// <summary>
    /// Button UI element
    /// </summary>
    public class UIButton
    {
        public Boolean visible;
        Game game;
        Texture2D buttonTex;
        Texture2D hoverTex;
        public Rectangle buttonBounds;
        public String text = "";

        SoundEffectInstance select;

        public UIButton(Game game, Texture2D buttonTex, Texture2D hoverTex, Vector2 position, String text)
        {
            visible = true;
            this.game = game;
            this.buttonTex = buttonTex;
            this.hoverTex = hoverTex;
            this.buttonBounds = new Rectangle((int)position.X, (int)position.Y, (int)buttonTex.Width, (int)buttonTex.Height);
            this.text = text;

            select = game.Content.Load<SoundEffect>("SoundEffects/select").CreateInstance();
        }

        public UIButton(Game game, Color color, Color hoverColor, Rectangle bounds, String text)
        {
            PixelTexture pix = new PixelTexture(game, color);
            PixelTexture hoverPix = new PixelTexture(game, hoverColor);

            visible = true;
            this.game = game;
            this.buttonTex = pix.Texture;
            this.hoverTex = hoverPix.Texture;
            this.buttonBounds = bounds;
            this.text = text;

            select = game.Content.Load<SoundEffect>("SoundEffects/select").CreateInstance();
        }

        public UIButton(Game game, Texture2D buttonTex, Texture2D hoverTex, Vector2 position)
        {
            visible = true;
            this.game = game;
            this.buttonTex = buttonTex;
            this.hoverTex = hoverTex;
            this.buttonBounds = new Rectangle((int)position.X, (int)position.Y, (int)buttonTex.Width, (int)buttonTex.Height);

            select = game.Content.Load<SoundEffect>("SoundEffects/select").CreateInstance();
        }

        public UIButton(Game game, Color color, Color hoverColor, Rectangle bounds)
        {
            PixelTexture pix = new PixelTexture(game, color);
            PixelTexture hoverPix = new PixelTexture(game, hoverColor);

            visible = true;
            this.game = game;
            this.buttonTex = pix.Texture;
            this.hoverTex = hoverPix.Texture;
            this.buttonBounds = bounds;

            select = game.Content.Load<SoundEffect>("SoundEffects/select").CreateInstance();
        }

        public void SetButtonColor(Color color)
        {
            PixelTexture pix = new PixelTexture(game, color);
            this.buttonTex = pix.Texture;
        }

        /// <summary>
        /// Is the Mouse hovering over the Button
        /// </summary>
        private Boolean Hovered()
        {
            if (!visible)
            {
                return false;
            }

            return buttonBounds.Contains(new Point(Mouse.GetState().X, Mouse.GetState().Y));
        }

        /// <summary>
        /// Has the user clicked the Button
        /// </summary>
        public virtual Boolean Clicked()
        {
            if (!visible)
            {
                return false;
            }

            if (((EngineGame)game).input.LeftClick())
            {
                return buttonBounds.Contains(new Point(Mouse.GetState().X, Mouse.GetState().Y));
            }

            return false;
        }

        /// <summary>
        /// Has the user released the Button
        /// </summary>
        public virtual Boolean Released()
        {
            if (!visible)
            {
                return false;
            }

            if (((EngineGame)game).input.LeftRelease())
            {
                Boolean activated = Hovered();

                if (activated && !((EngineGame)game).isMuted)
                {
                    select.Play();
                }

                return activated;
            }

            return false;
        }

        public virtual void Update(GameTime gameTime)
        {
        }

        public virtual void Draw(SpriteBatch sb, GameTime gameTime)
        {
            if (visible)
            {
                if (Hovered())
                    sb.Draw(hoverTex, buttonBounds, Color.White);
                else
                    sb.Draw(buttonTex, buttonBounds, Color.White);

                Vector2 textSize = ((EngineGame)game).font.MeasureString(text);
                sb.DrawString(((EngineGame)game).font, text, new Vector2(buttonBounds.X, buttonBounds.Y) + new Vector2(buttonBounds.Width / 2, buttonBounds.Height / 2) -
                    textSize / 2, Color.Black);
            }
        }
    }
}
