﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Engine;
using Microsoft.Kinect;
using System.Windows.Forms;
using Keys = Microsoft.Xna.Framework.Input.Keys;

namespace kinectApp
{
    public enum TRANSLATE_LOC
    {
        end,
        start,
        none
    };

    public class BranchCreator
    {
        Game1 game;

        public TreeBranch selectedBranch = null;
        public TreeNode selectedNode = null;

        public Boolean isCreatingBranches = false;

        LineManager tempBranchManager;
        public LineManager BranchManager;
        public TreeHierarchy treeHierarchy;

        TRANSLATE_LOC translateLoc = TRANSLATE_LOC.none;
        Tree tree;

        public BranchCreator(Game game)
        {
            this.game = (Game1)game;
            tempBranchManager = new LineManager(game);
            BranchManager = new LineManager(game);
            treeHierarchy = new TreeHierarchy(game);
        }

        public void SetSelectedNode(TreeNode node)
        {
            if (node != null)
            {
                selectedBranch = treeHierarchy.GetBranchById(Convert.ToInt32(node.Text.Split('_')[1]));
                selectedNode = node;

                Console.WriteLine("Selected Node: " + selectedBranch.id);
            }
            else
            {
                selectedBranch = null;
                selectedNode = null;
            }

            ResetBranchManager();
        }

        public void DeselectBranches()
        {
            selectedBranch = null;
            selectedNode = null;
            ResetBranchManager();
        }

        public void TranslateBranches(Vector3 moveToolPosition, TRANSLATE_LOC transType)
        {
            if (selectedBranch != null)
            {
                if (transType == TRANSLATE_LOC.end)
                {
                    Vector3 dif = moveToolPosition - selectedBranch.end;
                    selectedBranch.end = moveToolPosition;

                    foreach (TreeBranch branch in selectedBranch.children)
                    {
                        branch.start = Vector3.Lerp(selectedBranch.start, selectedBranch.end, branch.lerpAmount);
                        branch.end += dif;

                        foreach (TreeBranch child in treeHierarchy.GetAllChildren(branch))
                        {
                            child.start += dif;
                            child.end += dif;
                        }
                    }
                }
                else if (transType == TRANSLATE_LOC.start)
                {
                    if (selectedBranch.parent != null)
                    {
                        float l = Vector3.Distance(moveToolPosition, selectedBranch.parent.start);
                        float parentLength = Vector3.Distance(selectedBranch.parent.start,
                            selectedBranch.parent.end);

                        float amount = MathHelper.Clamp(l / parentLength, 0, 1);
                        Vector3 pos = Vector3.Lerp(selectedBranch.parent.start,
                            selectedBranch.parent.end, amount);

                        selectedBranch.lerpAmount = amount;

                        selectedBranch.start = pos;
                        game.moveTool.Position = pos;
                    }
                    else
                    {
                        selectedBranch.start = moveToolPosition;

                        foreach (TreeBranch branch in selectedBranch.children)
                        {
                            branch.start = Vector3.Lerp(selectedBranch.start, selectedBranch.end, branch.lerpAmount);
                        }
                    }
                }

                ResetBranchManager();
            }
        }

        public void ResetBranchManager()
        {
            BranchManager.ClearLines();

            List<TreeBranch> allChildren = new List<TreeBranch>();
            if (selectedBranch != null)
            {
                allChildren = treeHierarchy.GetAllChildren(selectedBranch);
            }

            foreach (TreeBranch branch in treeHierarchy.GetAllBranches())
            {
                if (branch == selectedBranch)
                {
                    BranchManager.AddLine(new Engine.Line(branch.start, branch.end), Color.Yellow);
                }
                else if (allChildren.Contains(branch))
                {
                    BranchManager.AddLine(new Engine.Line(branch.start, branch.end), Color.Blue);
                }
                else
                {
                    BranchManager.AddLine(new Engine.Line(branch.start, branch.end));
                }
            }
        }

        Vector3 clickPos = Vector3.Zero;
        Vector3 dragPos;
        Boolean isDraggingBranch = false;
        public void Update()
        {
            #region move
            if (selectedBranch != null)
            {
                if (game.input.Click(Keys.W) && !game.input.RightPressed())
                {
                    if (translateLoc == TRANSLATE_LOC.none)
                    {
                        translateLoc = TRANSLATE_LOC.end;
                        game.moveTool.isMoving = true;
                    }
                    else if (translateLoc == TRANSLATE_LOC.end)
                    {
                        translateLoc = TRANSLATE_LOC.start;
                        game.moveTool.isMoving = true;
                    }
                    else if (translateLoc == TRANSLATE_LOC.start)
                    {
                        translateLoc = TRANSLATE_LOC.none;
                        game.moveTool.isMoving = false;
                    }

                    if (translateLoc == TRANSLATE_LOC.end)
                    {
                        game.moveTool.Position = game.branchCreator.selectedBranch.end;
                    }
                    else if (translateLoc == TRANSLATE_LOC.start)
                    {
                        game.moveTool.Position = game.branchCreator.selectedBranch.start;
                    }
                }

                if (translateLoc != TRANSLATE_LOC.none)
                {
                    TranslateBranches(game.moveTool.Position, translateLoc);
                }
            }
            #endregion

            #region create branch
            if (!game.moveTool.isMoving)
            {
                if (game.input.LeftRelease())
                {
                    if (isDraggingBranch)
                    {
                        isDraggingBranch = false;
                        if (selectedBranch != null)
                        {
                            BranchManager.AddLine(new Line(selectedBranch.end, dragPos));
                        }
                        else
                        {
                            BranchManager.AddLine(new Line(clickPos, dragPos));
                        }
                        tempBranchManager.ClearLines();

                        TreeBranch newSelected;

                        if (selectedBranch != null)
                        {
                            newSelected = new TreeBranch(game, selectedBranch, selectedBranch.end, dragPos, BranchType.Trunk);
                            selectedBranch.children.Add(newSelected);
                            //tempBranchManager.AddLine(new Line(selectedBranch.end, dragPos));
                        }
                        else
                        {
                            newSelected = new TreeBranch(game, null, clickPos, dragPos, BranchType.Trunk);
                            treeHierarchy.AddBranch(newSelected);
                        }

                        selectedBranch = newSelected;

                        game.parentForm.branchViewWindow.RefreshView();
                        ResetBranchManager();
                    }
                }

                if (game.input.LeftClick() && game.input.inBounds())
                {
                    if (isCreatingBranches)
                    {
                        clickPos = GetClickPosition();
                        dragPos = clickPos;
                        isDraggingBranch = true;
                    }
                }

                if (isDraggingBranch)
                {
                    dragPos = GetClickPosition();
                    tempBranchManager.ClearLines();
                    if (selectedBranch != null)
                    {
                        tempBranchManager.AddLine(new Line(selectedBranch.end, dragPos));
                    }
                    else
                    {
                        tempBranchManager.AddLine(new Line(clickPos, dragPos));
                    }
                }
            }
            #endregion

            #region delete
            if (game.input.Click(Keys.Delete))
            {
                game.moveTool.isMoving = false;
                if (game.branchCreator.selectedNode != null) // treeWindow.branchTreeView.SelectedNode != null)
                {
                    game.parentForm.branchViewWindow.branchTreeView.Nodes.Remove(game.branchCreator.selectedNode); //game.selectedTreeNode);
                    treeHierarchy.RemoveBranch(game.branchCreator.selectedBranch.id);// Convert.ToInt32(game.selectedTreeNode.Text.Split('_')[1]));
                    game.branchCreator.DeselectBranches();
                    game.parentForm.branchViewWindow.branchTreeView.Refresh();
                    game.branchCreator.BranchManager.ClearLines();
                    foreach (TreeBranch branch in treeHierarchy.GetAllBranches())
                    {
                        game.branchCreator.BranchManager.AddLine(new Engine.Line(branch.start, branch.end));
                    }
                }
                else if (selectedBranch != null)
                {
                    treeHierarchy.RemoveBranch(selectedBranch.id);
                    game.branchCreator.DeselectBranches();
                    game.parentForm.branchViewWindow.branchTreeView.Refresh();
                    game.branchCreator.BranchManager.ClearLines();
                    foreach (TreeBranch branch in treeHierarchy.GetAllBranches())
                    {
                        game.branchCreator.BranchManager.AddLine(new Engine.Line(branch.start, branch.end));
                    }
                }
            }
            #endregion
            treeHierarchy.Update();
        }

        /// <summary>
        /// default to get location on point cloud, otherwise, interact with plane 5 units ahead
        /// </summary>
        /// <returns></returns>
        private Vector3 GetClickPosition()
        {
            if (game.kinectManager.pointCloud != null && game.kinectManager.showPointCloud)
            {
                Matrix View = 
                    Matrix.CreateLookAt(game.camera.Position, 
                    MouseRay.GetPointAlongSelectionRay(game, game.input.mouseLocation, game.camera, 5),
                    Vector3.Up);

                Matrix Projection = 
                    Matrix.CreatePerspectiveFieldOfView((float)((float)Math.PI / 50.0f), game.GraphicsDevice.Viewport.AspectRatio,
                    game.camera.NearPlaneDistance, game.camera.FarPlaneDistance);
                BoundingFrustum narrowClickFrustum = new BoundingFrustum(View * Projection);

                foreach (Vector3 point in game.kinectManager.pointCloud.points)
                {
                    if (narrowClickFrustum.Contains(point) != ContainmentType.Disjoint)
                    {
                        //Vector3 n = game.camera.Position - point;
                        //n.Normalize();

                        //float d = Vector3.Distance(game.camera.Position, point);
                        //Plane p = new Plane(n, d);

                        //return Rays.CalculateWorldFromVectorWithPlane(game.input.mouseLocation, game.GraphicsDevice, game.camera, p);

                        return point;
                    }
                }
            }

            return MouseRay.GetPointAlongSelectionRay(game, game.input.mouseLocation, game.camera, 5);
        }

        public void Reset()
        {
            tempBranchManager.ClearLines();
            BranchManager.ClearLines();
            foreach (TreeBranch branch in treeHierarchy.GetAllBranches())
            {
                treeHierarchy.RemoveBranch(branch.id);
            }

            treeHierarchy.resetBranches();
            game.parentForm.branchViewWindow.RefreshView();
            ResetBranchManager();

            selectedBranch = null;
            selectedNode = null;

            isCreatingBranches = false;
        }

        public void Draw()
        {
            tempBranchManager.Draw(game.camera);
            BranchManager.Draw(game.camera);
        }

        //grow
        public void createBranches(TreeBranch parent, int iterations)
        {
        /*    iterations--;
            if (iterations > 0)
            {
                int numBranches = 2; //TODO: base on rules
                List<TreeBranch> newBranches = new List<TreeBranch>();
                for (int i = 0; i < numBranches; i++)
                {
                    //float? angle = treeRules.GetValueFromVariation(360.0f * (float)((i + 1) / (float)numBranches), 0);
                    TreeBranch node = new TreeBranch(game, parent, parent.GetPositionOnBranch(), this.game.TreeRules().Length());
                    parent.AddChild(node);
                    newBranches.Add(node);
                }

                foreach (TreeBranch node in newBranches)
                {
                    createBranches(node, iterations);
                }
            }*/
        }
    }
}
