using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Engine;
using Microsoft.Kinect;
using System.Windows.Forms;
using Keys = Microsoft.Xna.Framework.Input.Keys;

namespace kinectApp
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        public Form GameWindow;
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        public FPSCamera camera;
        public Input input;

        public List<Tree> tree = new List<Tree>();

        public Form gameWindow;
        public MDIParent1 parentForm;

        public MoveTool moveTool;

        public BranchCreator branchCreator;
        public LeafCreator leafCreator;
        public KinectManager kinectManager;

        public Random rand;

        private TreeRules treeRules;
        //private PineTree pineTree;
        //private NotPineTree notPineTree;
        private CherryTree cherryTree;
        private BlackOak blackOak;
        private BalsamFir balsamFir;

        public Boolean showBoundingVolumes = true;

        public Boolean drawLeaves = true;

        private Dictionary<string, TreeRules> treeRulesDictionary = new Dictionary<string, TreeRules>();

        public int treeTypeIndex = 0;

        #region leaf textures
        public Texture2D leafTexture;
        public Texture2D pineNeedlesTexture;
        public Texture2D cherryBlossom;
        public Texture2D bark;
        public Texture2D blackOakLeaf;
        public Texture2D balsamFirTexture;
        #endregion

        private Tree activeTree;

        public Tree GetActiveTree()
        {
            return activeTree;
        }

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            //pineTree = new PineTree(this);
            //notPineTree = new NotPineTree(this);
            cherryTree = new CherryTree(this);
            blackOak = new BlackOak(this);
            balsamFir = new BalsamFir(this);

            //treeRulesDictionary.Add("Pine Tree", pineTree);
            //treeRulesDictionary.Add("Not Pine Tree", notPineTree);
            treeRulesDictionary.Add("Cherry Tree", cherryTree);
            treeRulesDictionary.Add("Black Oak", blackOak);
            treeRulesDictionary.Add("Balsam Fir", balsamFir);

            tree = new List<Tree>();
        }

        public List<String> treeRulesList()
        {
            return treeRulesDictionary.Keys.ToList<String>();
        }

        public TreeRules TreeRules()
        {
            return treeRulesDictionary[treeRulesList()[treeTypeIndex]];
        }

        protected override void Initialize()
        {
            rand = new Random(1);
            if (GameWindow != null)
            {
                GameWindow.Location = new System.Drawing.Point(0, 0);
                GameWindow.StartPosition = FormStartPosition.Manual;
            }
            IsMouseVisible = true;

            RasterizerState rState = new RasterizerState();
            rState.CullMode = CullMode.None;
            GraphicsDevice.RasterizerState = rState; 

            camera = new FPSCamera(this);
            Components.Add(camera);
            camera.Angles = new Vector2(0, 3.14f);

            input = new Input(this, gameWindow);
            Components.Add(input);

            branchCreator = new BranchCreator(this);
            leafCreator = new LeafCreator(this);
            kinectManager = new KinectManager(this);

            moveTool = new MoveTool(this, camera);

            base.Initialize();
        }

        public void Reset()
        {
            branchCreator.Reset();
            leafCreator.Reset();
            kinectManager.Reset();

            //leafQuads = new List<LeafTexture>();

            //branchVertices = new List<VertexPositionNormal>();
            //branchIndices = new List<int>();

            //if (branchBasicEffect != null)
            //{
            //    branchBasicEffect.Dispose();
            //    branchBasicEffect = null;
            //}

            //if (branchVertexBuffer != null)
            //{
            //    branchVertexBuffer.Dispose();
            //    branchVertexBuffer = null;
            //}

            //if (branchIndexBuffer != null)
            //{
            //    branchIndexBuffer.Dispose();
            //    branchIndexBuffer = null;
            //}

            tree = new List<Tree>();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            this.graphics.IsFullScreen = true;

            leafTexture = Content.Load<Texture2D>("leaf");
            pineNeedlesTexture = Content.Load<Texture2D>("pineNeedlesTexture");
            cherryBlossom = Content.Load<Texture2D>("cherryBlossom");
            bark = Content.Load<Texture2D>("bark");
            blackOakLeaf = Content.Load<Texture2D>("blackOakLeaf");
            balsamFirTexture = Content.Load<Texture2D>("balsamFir");
        }

        protected override void UnloadContent()
        {
            kinectManager.UnloadContent();
        }

        protected override void Update(GameTime gameTime)
        {
            branchCreator.Update();
            leafCreator.Update();
            kinectManager.Update();

            if (input.Click(Keys.T))
            {
                CreateTree();
            }

            if (input.Click(Keys.R))
            {
                if (tree != null && tree.Count > 0)
                {
                    foreach (Tree t in tree)
                    {
                        t.ResetLobes(branchCreator);
                    }
                }
            }

            if (tree != null && tree.Count > 0)
            {
                foreach (Tree t in tree)
                {
                    t.Update();
                }
            }

            moveTool.Update(gameTime);

            if (input.Click(Keys.O))
            {
                branchCreator.Reset();
                leafCreator.Reset();

                //game.tree.ResetLobes(game.branchCreator);
                branchCreator.BranchManager.AddLine(new Line(Vector3.Zero, Vector3.Up / 2));
                TreeBranch branch = new TreeBranch(this, null, Vector3.Zero, Vector3.Up / 2, BranchType.Trunk);
                branchCreator.treeHierarchy.AddBranch(branch);

                leafCreator.spheres.Add(new LeafLobe(this, new BoundingSphere(Vector3.Up, 1)));

                parentForm.branchViewWindow.RefreshView();
                branchCreator.ResetBranchManager();

                CreateTree();
            }

            if (input.Click(Keys.P))
            {
                for (int i = 0; i < 5; i++)
                {
                    branchCreator.Reset();
                    leafCreator.Reset();

                    //game.tree.ResetLobes(game.branchCreator);
                    branchCreator.BranchManager.AddLine(new Line(Vector3.Zero, Vector3.Up / 2));
                    TreeBranch branch = new TreeBranch(this, null, Vector3.Zero, Vector3.Up / 2, BranchType.Trunk);
                    branchCreator.treeHierarchy.AddBranch(branch);

                    leafCreator.spheres.Add(new LeafLobe(this, new BoundingSphere(Vector3.Up, 1)));

                    parentForm.branchViewWindow.RefreshView();
                    branchCreator.ResetBranchManager();

                    CreateTree();
                }
            }

            base.Update(gameTime);
        }

        public void CreateTree()
        {
            Console.WriteLine("Creating Tree");

            Tree t = new Tree(this, branchCreator, kinectManager.pointCloud, false);
            activeTree = t;
            tree.Add(t);
            t.Initialize();
            t.CreateGeometry();
        }

        public Vector3 GetRandomPointOnSphere(BoundingSphere sphere, float minY = 0, float maxY = 1)
        {
            float theta = (float)rand.NextDouble() * (float)(Math.PI * 2);
            float u = ((float)rand.NextDouble() * 2.0f) - 1.0f;

            Vector3 point = new Vector3((float)Math.Cos(theta) * (float)Math.Sqrt((1 - u * u)),
                ((float)Math.Sin(theta) * (float)Math.Sqrt(1 - u * u)), u);

            Vector3 loc = sphere.Center + (point * sphere.Radius);

            loc.Y = MathHelper.Clamp(loc.Y, minY, maxY);

            Vector3 dir = sphere.Center - loc;
            dir.Normalize();

            return sphere.Center + dir * sphere.Radius;
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            //spriteBatch.Begin();

            moveTool.Draw(gameTime);
            if (showBoundingVolumes)
            {
                branchCreator.Draw();
            }
            leafCreator.Draw();
            kinectManager.Draw();

            if (tree != null && tree.Count > 0)
            {
                foreach (Tree t in tree)
                {
                    t.Draw();
                }
            }

            //spriteBatch.End();
            base.Draw(gameTime);
        }
    }
}
