﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Engine;
using Microsoft.Kinect;

namespace kinectApp
{
    public class KinectManager
    {
        Game1 game;
        public Kinect kinect;
        public PointCloud pointCloud;

        bool constantUpdate = true;

        int sampleRate = 5;

        public float minX = -1;
        public float maxX = 5;
        public float minY = -1;
        public float maxY = float.MaxValue;
        public float minZ = 0;
        public float maxZ = 3;

        public Boolean showPointCloud = true;

        public KinectManager(Game game)
        {
            this.game = (Game1)game;
            kinect = new Kinect(game);
            Initialize();
        }

        public void Reset()
        {
            UnloadContent();
            Initialize();
        }

        public void Initialize()
        {
            minX = -1;
            maxX = 5;
            minY = -1;
            maxY = float.MaxValue;
            minZ = 0;
            maxZ = 3;

            sampleRate = 5;
            constantUpdate = true;
            pointCloud = null;
        }

        public void UnloadContent()
        {
            kinect.StopSensor();
            kinect.UnloadContent();
        }

        public void Update()
        {
            if (kinect.isKinectOn())
            {
                if (game.input.Click(Keys.OemMinus))
                {
                    sampleRate = (int)MathHelper.Clamp(sampleRate + 1, 0, 100);
                }

                if (game.input.Click(Keys.OemPlus))
                {
                    sampleRate = (int)MathHelper.Clamp(sampleRate - 1, 0, 100);
                }

                if (game.input.Click(Keys.Space))
                {
                    constantUpdate = !constantUpdate;
                }

                if (game.input.Click(Keys.Enter) || (game.input.RightPressed() && game.input.LeftPressed()))
                {
                    constantUpdate = false;
                    pointCloud = kinect.GetPointCloud(mod: 1, minX: minX, maxX: maxX, minY: minY, maxY: maxY, minZ: minZ, maxZ: maxZ);
                }

                if (constantUpdate)
                {
                    pointCloud = kinect.GetPointCloud(mod: sampleRate, minX: minX, maxX: maxX, minY: minY, maxY: maxY, minZ: minZ, maxZ: maxZ);
                }

                if (game.input.Click(Keys.Tab))
                {
                    kinect.ToggleDistance();
                }
            }
        }

        public void EnableKinect()
        {
            kinect.StartSensor();
        }

        public void DisableKinect()
        {
            kinect.StopSensor();
        }

        public void RotateUp()
        {
            kinect.RotateUp();
        }

        public void RotateDown()
        {
            kinect.RotateDown();
        }

        public void Draw()
        {
            if (pointCloud != null && showPointCloud)
            {
                pointCloud.Draw(game.camera);
            }
        }
    }
}
