﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Engine;

namespace kinectApp
{
    public class LeafCreator
    {
        Game1 game;
        public List<LeafLobe> spheres = new List<LeafLobe>();

        public Boolean isCreatingLeaves = false;

        public LeafLobe selectedLeafLobe = null;

        public LeafCreator(Game game)
        {
            this.game = (Game1)game;
        }

        public void Update()
        {
            if (selectedLeafLobe != null)
            {
                if (game.input.Click(Keys.Down))
                {
                    selectedLeafLobe.sphere.Radius -= 0.1f;
                }
                if (game.input.Click(Keys.Up))
                {
                    selectedLeafLobe.sphere.Radius += 0.1f;
                }
            }
            if (game.input.Click(Keys.Delete))
            {
                game.moveTool.isMoving = false;
                if (selectedLeafLobe != null)
                {
                    spheres.Remove(selectedLeafLobe);
                }
            }

            if (selectedLeafLobe != null)
            {
                if (game.input.IsKeyDown(Keys.LeftControl) || game.input.IsKeyDown(Keys.RightControl))
                {
                    if (game.input.Click(Keys.D))
                    {
                        //Duplicate Selected Leaf Node
                        spheres.Add(new LeafLobe(game, new BoundingSphere(selectedLeafLobe.sphere.Center, selectedLeafLobe.sphere.Radius)));
                    }
                }
            }

            #region select
            if (game.input.LeftClick())
            {
                if (!game.moveTool.isMoving)
                {
                    Ray ray = Engine.Rays.CalculateCursorRay(game.input.mouseLocation, game.GraphicsDevice, game.camera);

                    float closestDist = float.MaxValue;
                    LeafLobe closestSphere = null;

                    foreach (LeafLobe sphere in spheres)
                    {
                        float? dist = sphere.sphere.Intersects(ray);

                        if (dist != null)
                        {
                            if ((float)dist < closestDist)
                            {
                                closestSphere = sphere;
                                closestDist = (float)dist;
                            }
                        }
                    }

                    selectedLeafLobe = closestSphere;
                }
            }
            #endregion

            #region move
            if (selectedLeafLobe != null)
            {
                if (game.input.Click(Keys.W) && !game.input.RightPressed())
                {
                    game.moveTool.isMoving = !game.moveTool.isMoving;
                    game.moveTool.Position = selectedLeafLobe.sphere.Center;
                }

                if (game.moveTool.isMoving)
                {
                    selectedLeafLobe.sphere = new BoundingSphere(game.moveTool.Position,
                        selectedLeafLobe.sphere.Radius);
                }
            }
            #endregion

            #region create leaves
            if (!game.moveTool.isMoving && selectedLeafLobe == null)
            {
                if (game.input.LeftClick() && game.input.inBounds())
                {
                    if (isCreatingLeaves)
                    {
                        spheres.Add(new LeafLobe(game, new BoundingSphere(GetClickPosition()/*MouseRay.GetPointAlongSelectionRay(game, game.input.mouseLocation, game.camera, 5)*/, 1)));
                    }
                }
            }
            #endregion
        }

        /// <summary>
        /// default to get location on point cloud, otherwise, interact with plane 5 units ahead
        /// </summary>
        /// <returns></returns>
        private Vector3 GetClickPosition()
        {
            if (game.kinectManager.pointCloud != null && game.kinectManager.showPointCloud)
            {
                Matrix View =
                    Matrix.CreateLookAt(game.camera.Position,
                    MouseRay.GetPointAlongSelectionRay(game, game.input.mouseLocation, game.camera, 5),
                    Vector3.Up);

                Matrix Projection =
                    Matrix.CreatePerspectiveFieldOfView((float)((float)Math.PI / 50.0f), game.GraphicsDevice.Viewport.AspectRatio,
                    game.camera.NearPlaneDistance, game.camera.FarPlaneDistance);
                BoundingFrustum narrowClickFrustum = new BoundingFrustum(View * Projection);

                foreach (Vector3 point in game.kinectManager.pointCloud.points)
                {
                    if (narrowClickFrustum.Contains(point) != ContainmentType.Disjoint)
                    {
                        //Vector3 n = game.camera.Position - point;
                        //n.Normalize();

                        //float d = Vector3.Distance(game.camera.Position, point);
                        //Plane p = new Plane(n, d);

                        //return Rays.CalculateWorldFromVectorWithPlane(game.input.mouseLocation, game.GraphicsDevice, game.camera, p);

                        return point;
                    }
                }
            }

            return MouseRay.GetPointAlongSelectionRay(game, game.input.mouseLocation, game.camera, 5);
        }

        public void Reset()
        {
            spheres = new List<LeafLobe>();
            isCreatingLeaves = false;
            selectedLeafLobe = null;
            game.moveTool.isMoving = false;
        }

        public void Draw()
        {
            if (game.showBoundingVolumes)
            {
                foreach (LeafLobe sphere in spheres)
                {
                    if (selectedLeafLobe != null && sphere == selectedLeafLobe)
                    {
                        BoundingSphereRenderer.Render(sphere.sphere, game.GraphicsDevice, game.camera.View, game.camera.Projection, Color.Yellow);
                    }
                    else
                    {
                        BoundingSphereRenderer.Render(sphere.sphere, game.GraphicsDevice, game.camera.View, game.camera.Projection, Color.Green);
                    }
                }
            }
        }
    }
}
