﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Engine;

namespace kinectApp
{
    public partial class MDIParent1 : Form
    {
        private int childFormNumber = 0;
        Game1 game;

        //forms
        BoundForm boundForm;
        public TreeHierarchyForm branchViewWindow;

        public MDIParent1(Game1 game)
        {
            this.game = game;
            InitializeComponent();
            boundForm = new BoundForm(game);
            branchViewWindow = new TreeHierarchyForm(game);

            foreach (String key in game.treeRulesList())
            {
                treeTypeComboBox.Items.Add(key);
            }

            treeTypeComboBox.SelectedIndexChanged += treeTypeComboBox_SelectedIndexChanged;
        }

        private void treeTypeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            game.treeTypeIndex = treeTypeComboBox.SelectedIndex;
        }

        private void ShowNewForm(object sender, EventArgs e)
        {
            /*Form childForm = new Form();
            childForm.MdiParent = this;
            childForm.Text = "Window " + childFormNumber++;
            childForm.Show();*/

            game.Reset();
        }

        private void OpenFile(object sender, EventArgs e)
        {
            game.Reset();
            OpenDocument();
        }

        private void OpenDocument()
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            openFileDialog.Filter = "All Files|*.*|Point Cloud (*.xyz)|*.xyz|Tree (*.tree)|*.tree";
            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                string FileName = openFileDialog.FileName;

                if (FileName.EndsWith(".xyz"))
                {
                    loadPointCloud(FileName);
                }
                else if (FileName.EndsWith(".tree"))
                {
                    loadTree(FileName);
                }
            }
        }

        private void SaveDocument()
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog();
            saveFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            saveFileDialog.Filter = "Point Cloud (*.xyz)|*.xyz|Tree (*.tree)|*.tree";
            if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                string FileName = saveFileDialog.FileName;

                if (FileName.EndsWith(".xyz"))
                {
                    savePointCloud(FileName);
                }
                else if (FileName.EndsWith(".tree"))
                {
                    saveTree(FileName);
                }
            }
        }

        private void savePointCloud(string fileName)
        {
            Console.WriteLine(fileName);
            if (game.kinectManager.pointCloud != null)
            {
                using (StreamWriter writer = new StreamWriter(fileName))
                {
                    foreach (Vector3 point in game.kinectManager.pointCloud.points)
                    {
                        writer.WriteLine(point.X + " " + point.Y + " " + point.Z);
                    }
                }
            }
        }

        private void saveTree(string fileName)
        {
            if (game.kinectManager.pointCloud != null)
            {
                savePointCloud(fileName + ".xyz");
            }

            //game.branchCreator.treeHierarchy.
            using (StreamWriter writer = new StreamWriter(fileName))
            {
                writer.WriteLine("#BRANCHES");

                foreach (TreeBranch branch in game.branchCreator.treeHierarchy.ParentBranches())
                {
                    WriteBranch(writer, branch);
                    TraverseTreeForWrite(branch, writer);
                }

                writer.WriteLine("#LEAF LOBES");
                foreach (LeafLobe sphere in game.leafCreator.spheres)
                {
                    WriteLeafLobe(writer, sphere);
                }
            }
        }

        private void WriteLeafLobe(StreamWriter writer, LeafLobe lobe)
        {
            writer.WriteLine(lobe.sphere.Center.X + " " + lobe.sphere.Center.Y + " " + lobe.sphere.Center.Z + " " + lobe.sphere.Radius);
        }

        private void WriteBranch(StreamWriter writer, TreeBranch branch)
        {
            WriteBranch(writer,
                (branch.parent == null) ? -1 : branch.parent.id,
                branch.id,
                branch.start,
                branch.end,
                branch.lerpAmount);
        }

        private void WriteBranch(StreamWriter writer, int parentId, int id, Vector3 start, Vector3 end, float lerpAmount)
        {
            writer.WriteLine(id + " " + parentId + " " + start.X + " " + start.Y + " " + start.Z + " " + end.X + " " + end.Y + " " + end.Z + " " + lerpAmount);
        }

        private void TraverseTreeForWrite(TreeBranch parent, StreamWriter writer)
        {
            foreach (TreeBranch branch in parent.children)
            {
                WriteBranch(writer, branch);
                TraverseTreeForWrite(branch, writer);
            }
        }

        private void loadPointCloud(string fileName)
        {
            game.kinectManager.DisableKinect();
            this.kinectEnableButton.Checked = false;

            string[] lines = System.IO.File.ReadAllLines(fileName);
            List<Vector3> points = new List<Vector3>();

            foreach (string line in lines)
            {
                string[] parts = line.Split(' ');

                if (parts.Length == 3)
                {
                    Vector3? point = null;
                    try
                    {
                        point = new Vector3(
                            (float)(Convert.ToDouble(parts[0])),
                            (float)(Convert.ToDouble(parts[1])),
                            (float)(Convert.ToDouble(parts[2])));
                    }
                    catch { }
                    if (point != null)
                    {
                        points.Add((Vector3)point);
                    }
                }
            }

            Console.WriteLine(points.Count);
            game.kinectManager.pointCloud = new Engine.PointCloud(game, points);
        }

        List<TreeBranch> branches = new List<TreeBranch>();
        List<TreeBranch> allBranches = new List<TreeBranch>();
        List<LeafLobe> lobes = new List<LeafLobe>();
        private void loadTree(string fileName)
        {
            branches = new List<TreeBranch>();
            allBranches = new List<TreeBranch>();
            lobes = new List<LeafLobe>();

            int highestId = int.MinValue;

            if (File.Exists(fileName + ".xyz"))
            {
                loadPointCloud(fileName + ".xyz");
            }

            string[] lines = System.IO.File.ReadAllLines(fileName);

            Boolean loadBranches = false;
            Boolean loadLeaves = false;
            foreach (string line in lines)
            {
                if (line.Trim() == "#BRANCHES")
                {
                    loadBranches = true;
                    loadLeaves = false;
                }
                else if (line.Trim() == "#LEAF LOBES")
                {
                    loadLeaves = true;
                    loadBranches = false;
                }

                else if (loadBranches)
                {
                    TreeBranch branch;
                    string[] tokens = line.Split(' ');

                    //load branch
                    if (tokens.Length == 9)
                    {
                        if (Convert.ToInt32(tokens[1]) == -1)
                        {
                            branch = new TreeBranch(game, null,
                                new Vector3(
                                    (float)Convert.ToDouble(tokens[2]),
                                    (float)Convert.ToDouble(tokens[3]),
                                    (float)Convert.ToDouble(tokens[4])),
                                new Vector3(
                                    (float)Convert.ToDouble(tokens[5]),
                                    (float)Convert.ToDouble(tokens[6]),
                                    (float)Convert.ToDouble(tokens[7])), BranchType.Trunk);

                            branch.lerpAmount = (float)Convert.ToDouble(tokens[8]);
                            branch.id = Convert.ToInt32(tokens[0]);
                            allBranches.Add(branch);
                            branches.Add(branch);

                            if (branch.id > highestId)
                            {
                                highestId = branch.id;
                            }
                        }
                        else
                        {
                            TreeBranch parent = null;
                            int id = Convert.ToInt32(tokens[0]);
                            int parentId = Convert.ToInt32(tokens[1]);

                            foreach (TreeBranch b in allBranches)
                            {
                                if (b.id == parentId)
                                {
                                    parent = b;
                                    break;
                                }
                            }

                            branch = new TreeBranch(game, parent,
                                new Vector3(
                                    (float)Convert.ToDouble(tokens[2]),
                                    (float)Convert.ToDouble(tokens[3]),
                                    (float)Convert.ToDouble(tokens[4])),
                                new Vector3(
                                    (float)Convert.ToDouble(tokens[5]),
                                    (float)Convert.ToDouble(tokens[6]),
                                    (float)Convert.ToDouble(tokens[7])), BranchType.Trunk);

                            branch.lerpAmount = (float)Convert.ToDouble(tokens[8]);
                            branch.id = id;
                            parent.AddChild(branch);
                            allBranches.Add(branch);

                            if (branch.id > highestId)
                            {
                                highestId = branch.id;
                            }
                        }
                    }

                    int staticId = highestId + 1;
                    TreeBranch.SetStaticId(staticId);
                }

                else if (loadLeaves)
                {
                    string[] tokens = line.Split(' ');

                    if (tokens.Length == 4)
                    {
                        Vector3 pos = new Vector3(
                            (float)Convert.ToDouble(tokens[0]),
                            (float)Convert.ToDouble(tokens[1]),
                            (float)Convert.ToDouble(tokens[2]));

                        float rad = (float)Convert.ToDouble(tokens[3]);

                        BoundingSphere sphere = new BoundingSphere(pos, rad);

                        LeafLobe lobe = new LeafLobe(game, sphere);
                        lobes.Add(lobe);
                    }
                }
            }

            game.branchCreator.treeHierarchy.setBranches(branches);
            game.branchCreator.ResetBranchManager();
            //game.branchCreator.treeHierarchy.resetBranches();

            game.leafCreator.spheres = lobes;
        }

        private void SaveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveDocument();
        }

        private void ExitToolsStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void CutToolStripMenuItem_Click(object sender, EventArgs e)
        {
        }

        private void CopyToolStripMenuItem_Click(object sender, EventArgs e)
        {
        }

        private void PasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
        }

        private void ToolBarToolStripMenuItem_Click(object sender, EventArgs e)
        {
            toolStrip.Visible = toolBarToolStripMenuItem.Checked;
        }

        private void StatusBarToolStripMenuItem_Click(object sender, EventArgs e)
        {
            statusStrip.Visible = statusBarToolStripMenuItem.Checked;
        }

        private void CascadeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.Cascade);
        }

        private void TileVerticalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileVertical);
        }

        private void TileHorizontalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileHorizontal);
        }

        private void ArrangeIconsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.ArrangeIcons);
        }

        private void CloseAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            foreach (Form childForm in MdiChildren)
            {
                childForm.Close();
            }
        }

        private void kinectEnableButton_Click(object sender, EventArgs e)
        {
            if (kinectEnableButton.Checked)
            {
                game.kinectManager.EnableKinect();
            }
            else
            {
                game.kinectManager.DisableKinect();
            }

        }

        private void boundWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            boundForm = new BoundForm(game);
            //boundForm.MdiParent = this;
            boundForm.Text = "Set Kinect Range Bounds";

            boundForm.minXTextBox.Text = game.kinectManager.minX.ToString();
            boundForm.maxXTextBox.Text = game.kinectManager.maxX.ToString();
            boundForm.minYTextBox.Text = game.kinectManager.minY.ToString();
            boundForm.maxYTextBox.Text = game.kinectManager.maxY.ToString();
            boundForm.minZTextBox.Text = game.kinectManager.minZ.ToString();
            boundForm.maxZTextBox.Text = game.kinectManager.maxZ.ToString();

            boundForm.Show();
        }

        private void saveToolStripButton_Click(object sender, EventArgs e)
        {
            SaveDocument();
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveDocument();
        }

        private void distanceToggleButton_Click(object sender, EventArgs e)
        {
            game.kinectManager.kinect.ToggleDistance();
        }

        private void leafAreaCreatorButton_Click(object sender, EventArgs e)
        {
            game.moveTool.isMoving = false;
            game.branchCreator.selectedBranch = null;
            game.branchCreator.selectedNode = null;
            if (leafAreaCreatorButton.Checked)
            {
                DisableBranchCreator();
                game.leafCreator.isCreatingLeaves = true;
            }
            else
            {
                DisableLeafCreator();
            }
        }

        private void DisableLeafCreator()
        {
            leafAreaCreatorButton.Checked = false;
            game.leafCreator.isCreatingLeaves = false;
        }

        private void DisableBranchCreator()
        {
            branchCreatorButton.Checked = false;
            game.branchCreator.isCreatingBranches = false;
        }

        private void branchCreatorButton_Click(object sender, EventArgs e)
        {
            game.moveTool.isMoving = false;
            game.leafCreator.selectedLeafLobe = null;
            if (branchCreatorButton.Checked)
            {
                DisableLeafCreator();
                game.branchCreator.isCreatingBranches = true;
            }
            else
            {
                DisableBranchCreator();
            }
        }

        private void branchViewWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //branchViewWindow.MdiParent = this;
            branchViewWindow = new TreeHierarchyForm(game);
            branchViewWindow.Text = "Tree Hierarchy";
            branchViewWindow.Show();
        }

        private void togglePointCloudButton_Click(object sender, EventArgs e)
        {
            game.kinectManager.showPointCloud = togglePointCloudButton.Checked;
            game.showBoundingVolumes = !game.showBoundingVolumes;
        }

        private void rotateKinectUpButton_Click(object sender, EventArgs e)
        {
            game.kinectManager.RotateUp();
        }

        private void rotateKinectDownButton_Click(object sender, EventArgs e)
        {
            game.kinectManager.RotateDown();
        }

        private void generateTreeButton_Click(object sender, EventArgs e)
        {
            game.CreateTree();
            //game.tree.CreateGeometry();
            
            //Toggle Point Cloud
            togglePointCloudButton.Checked = false;
            game.kinectManager.showPointCloud = togglePointCloudButton.Checked;
            game.showBoundingVolumes = false;
        }

        private void resetTreeButton_Click(object sender, EventArgs e)
        {
            game.Reset();
            //if (game.tree != null)
            //{
                game.showBoundingVolumes = false;

                //for (int i = 0; i < 2; i++)
                //{
                    game.branchCreator.Reset();
                    game.leafCreator.Reset();

                    //game.tree.ResetLobes(game.branchCreator);
                    game.branchCreator.BranchManager.AddLine(new Line(Vector3.Zero, Vector3.Up / 2));
                    TreeBranch branch = new TreeBranch(game, null, Vector3.Zero, Vector3.Up / 2, BranchType.Trunk);
                    game.branchCreator.treeHierarchy.AddBranch(branch);

                    game.leafCreator.spheres.Add(new LeafLobe(game, new BoundingSphere(Vector3.Up, 1)));

                    game.parentForm.branchViewWindow.RefreshView();
                    game.branchCreator.ResetBranchManager();

                    game.CreateTree();
                //}
                //game.tree.CreateGeometry();
            //}
        }

        private void createGeometryButton_Click(object sender, EventArgs e)
        {
            //if (game.tree != null)
            //{
            //    game.tree.CreateGeometry();
            //}
        }

        private void drawLeavesButton_Click(object sender, EventArgs e)
        {
            game.drawLeaves = !game.drawLeaves;
        }
    }
}
