﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Engine;

namespace kinectApp
{
    public class MoveTool
    {
        Game1 game;
        public FPSCamera camera;
        public Vector3 Position = Vector3.Zero;
        private Matrix Scale;

        private Model Model;

        public Boolean isMoving = false;

        public Boolean isMovingX = false;
        public Boolean isMovingY = false;
        public Boolean isMovingZ = false;

        Matrix[] transforms;

        public MoveTool(Game game, FPSCamera cam)
            : base()
        {
            this.game = (Game1)game;
            camera = cam;

            Model = game.Content.Load<Model>("TOOL_move");
            transforms = new Matrix[Model.Bones.Count];
            Model.CopyAbsoluteBoneTransformsTo(transforms);

            Scale = Matrix.CreateScale(0.1f);
        }

        public Matrix World()
        {
            return Scale * Matrix.CreateTranslation(Position);
        }

        MouseState mouse;
        MouseState prevMouse;
        public void Update(GameTime gameTime)
        {
            prevMouse = mouse;
            mouse = Mouse.GetState();

            #region Moving
            if (isMoving)
            {
                //Plane to move against
                Vector3 normal = camera.Position - camera.Target;
                normal.Normalize();
                float d = Vector3.Distance(camera.Position, Position);

                Plane p = new Plane(normal, d); //Vector3.Up, Position.Y);

                //Scale = Matrix.CreateScale(Vector3.Distance(Position, camera.Position) / 50);

                if (game.input.LeftClick())
                {
                    Ray ray = Rays.CalculateCursorRay(game.input.mouseLocation, game.GraphicsDevice, camera);
                    ModelMesh mesh = Rays.RayIntersectsMesh(camera, ray, Model, World());

                    if (mesh != null)
                    {
                        if (mesh.Name.Contains("MOVE_X"))
                        {
                            isMovingX = true;
                        }
                        else if (mesh.Name.Contains("MOVE_Y"))
                        {
                            isMovingY = true;
                        }
                        else if (mesh.Name.Contains("MOVE_Z"))
                        {
                            isMovingZ = true;
                        }
                    }
                }

                if (mouse.LeftButton == ButtonState.Released)
                {
                    isMovingX = false;
                    isMovingY = false;
                    isMovingZ = false;
                }

                if (isMovingX)
                {
                    Position.X += Rays.WorldPositionOnTickWithPlane(game.input.mouseLocation, game.input.prevMouseLocation, game.GraphicsDevice, camera, p).X;
                }

                if (isMovingY)
                {
                    Position.Y += Rays.WorldPositionOnTickWithPlane(game.input.mouseLocation, game.input.prevMouseLocation, game.GraphicsDevice, camera, p).Y;
                }

                if (isMovingZ)
                {
                    Position.Z += Rays.WorldPositionOnTickWithPlane(game.input.mouseLocation, game.input.prevMouseLocation, game.GraphicsDevice, camera, p).Z;
                }
            }
            #endregion
        }

        public void Draw(GameTime gameTime)
        {
            if (isMoving)
            {
                ModelMethods.DrawBasicModel(camera.Projection,
                    camera.View, Model, World(), transforms);
            }
        }
    }
}