﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Engine;

namespace kinectApp
{
    //TODO: read from XML
    public class BlackOak : TreeRules
    {
        Game1 game;

        public BlackOak(Game1 game)
            : base()
        {
            this.game = game;
        }

        private int numPointsOnLobe = 15;

        private int angle = 45; //angle of branches
        private int angleVariation = 0;

        private int trunkAngle = 10;

        public int TrunkAngle()
        {
            return trunkAngle;
        }

        private float leafTaper = 0;

        public float LeafTaper()
        {
            return leafTaper;
        }

        private int numIterations = 1;

        public int NumIterations()
        {
            return numIterations;
        }

        public int BranchIterations()
        {
            return 3;
        }

        private int subsequentAngle = 15;

        public int SubsequentAngle()
        {
            return subsequentAngle;
        }

        private int numFinalBranches = 4;

        private int numFinalBranchesVariation = 2;

        private int numBranchesOffSplit = 1;

        public int NumTrunkBranchesOffSplit()
        {
            return numBranchesOffSplit;
        }

        public int NumBranchesOffSplit()
        {
            return 2;
        }

        private float numSegmentsInBranch = 25;

        public float NumSegmentsInBranch()
        {
            return numSegmentsInBranch;
        }

        //offset as percentage of segment length
        private float segmentOffset = 0.4f;

        public float SegmentOffset()
        {
            return segmentOffset;
        }

        public int NumFinalBranches()
        {
            return 0;// (int)this.GetValueFromVariation((float)numFinalBranches, (float)numFinalBranchesVariation);
        }

        private float lengthReductionFactor = 0.9f;

        public float LengthReductionFactor()
        {
            return lengthReductionFactor;
        }

        private float branchStartWidth = 0.1f;

        public float BranchStartWidth()
        {
            return branchStartWidth;
        }

        private float trunkStartWidth = 0.2f;
        public float TrunkStartWidth()
        {
            return trunkStartWidth;
        }

        private float trunkTaperPercent = 0.4f;
        public float TrunkTaperPercent()
        {
            return trunkTaperPercent;
        }

        public float MaxTrunkHeightPercent()
        {
            return 0.5f;
        }

        private float minBranchWidth = 0.005f;

        public float MinBranchWidth()
        {
            return minBranchWidth;
        }

        private float taperPercent = 0.05f;

        public float branchSphereConnectorScaleFactor()
        {
            return 0.95f;
        }

        public float TaperPercent()
        {
            return taperPercent;
        }

        public int NumPointsOnLobe()
        {
            return numPointsOnLobe;
        }

        int numLeafPointsOnLobe = 30;
        public int NumLeafPointsOnLobe()
        {
            return numLeafPointsOnLobe;
        }

        public float Angle()
        {
            float a = GetValueFromVariation(angle, angleVariation);
            return a;
        }

        //clamp on [value - variation : value + variation]
        public float GetValueFromVariation(float value, float variation)
        {
            return value + ((float)game.rand.NextDouble() * (variation * 2)) - variation;
        }

        public void CreateLeaves(Game1 game, TreeBranch branch)
        {
        }

        public Texture2D LeafTexture()
        {
            return game.blackOakLeaf;
        }

        public List<LeafLobe> CreateBranchControlPoints(List<LeafLobe> leafLobes)
        {
            foreach (LeafLobe lobe in leafLobes)
            {
                lobe.controlPoints = new List<BoundingSphere>();
                for (int i = 0; i < NumPointsOnLobe(); i++)
                {
                    Vector3 point = game.GetRandomPointOnSphere(lobe.sphere, -3.0f, 0.5f);
                    lobe.controlPoints.Add(new BoundingSphere(point, 0.1f));
                }
            }

            return leafLobes;
        }

        public List<LeafLobe> CreateLeafControlPoints(List<LeafLobe> leafLobes)
        {
            foreach (LeafLobe lobe in leafLobes)
            {
                lobe.leafControlPoints = new List<BoundingSphere>();
                for (int i = 0; i < NumLeafPointsOnLobe(); i++)
                {
                    Vector3 point = game.GetRandomPointOnSphere(lobe.sphere);
                    lobe.leafControlPoints.Add(new BoundingSphere(point, 0.1f));
                }
            }

            return leafLobes;
        }

        private int branchIterationFactor = 10;
        public void ConnectBranchesToLeafLobes()
        {
            List<TreeBranch> branches = new List<TreeBranch>();
            foreach (TreeBranch branch in game.branchCreator.treeHierarchy.GetAllBranches())
            {
                if (branch.branchType == BranchType.Branch)
                {
                    branches.Add(branch);
                }
            }

            for (int i = 0; i < branches.Count; i += branchIterationFactor)
            {
                TreeBranch branch = branches[i];

                Vector3 dir = Vector3.Cross(Vector3.Up, branch.Dir());
                dir.Normalize();

                //TreeBranch child = new TreeBranch(game, branch, branch.start, branch.start + (dir * 0.05f), BranchType.Twig);
                //branch.AddChild(child);

                AddLeaves(branch);
            }

            game.parentForm.branchViewWindow.RefreshView();
            game.branchCreator.ResetBranchManager();
        }

        private float leafDim = 0.175f;
        private void AddLeaves(TreeBranch child, float offset = 0.0f, int iterations = 2)
        {
            Vector3 parentDir = parentDir = child.parent.Dir();

            Vector3 lateral = Vector3.Cross(child.Dir(), parentDir);
            lateral.Normalize();

            Vector3 s = child.start + child.Dir() * (offset * GetTwigLength());
            Vector3 e = s + lateral * GetTwigLength();
            e += child.Dir();// *flowerTwigDirOffset;

            Vector3 dir = e - s;
            dir.Normalize();

            e = s + dir * GetTwigLength();

            TreeBranch leafTwig = new TreeBranch(game, child, s, e, BranchType.Twig);
            child.AddChild(leafTwig);

            Quad leaf = new Quad(game, leafTwig.end, leafTwig.Dir(), Vector3.Up, leafDim, leafDim);
            leafTwig.tree.AddLeaf(leaf, LeafTexture());

            if (iterations - 1 > 0)
            {
                iterations--;
                AddLeaves(leafTwig, 0.5f, iterations);
            }
        }

        private float GetTwigLength()
        {
            return 0.1f;
        }

        public float TwigStartWidth()
        {
            return 0.005f;
        }

        public int Tesselation()
        {
            return 5;
        }
    }
}
