﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Engine;

namespace kinectApp
{
    //TODO: read from XML
    public class CherryTree : TreeRules
    {
        Game1 game;

        public CherryTree(Game1 game)
            : base()
        {
            this.game = game;
        }

        private int numPointsOnLobe = 6;

        private int angle = 45; //angle of branches
        private int angleVariation = 0;

        private int trunkAngle = 3;

        public int TrunkAngle()
        {
            return trunkAngle;
        }

        private float leafTaper = 0;

        public float LeafTaper()
        {
            return leafTaper;
        }

        private int numIterations = 2;

        public int NumIterations()
        {
            return numIterations;
        }

        public int BranchIterations()
        {
            return 1;
        }

        private int subsequentAngle = 15;

        public int SubsequentAngle()
        {
            return subsequentAngle;
        }

        private int numFinalBranches = 4;

        private int numFinalBranchesVariation = 2;

        private int numBranchesOffSplit = 2;
        private int minNumBranchesOffSplit = 1;

        public int NumTrunkBranchesOffSplit()
        {
            return game.rand.Next(minNumBranchesOffSplit, numBranchesOffSplit + 1);
        }

        public int NumBranchesOffSplit()
        {
            return 1;
        }

        public float branchSphereConnectorScaleFactor()
        {
            return 0.82f;
        }

        private float numSegmentsInBranch = 4;

        public float NumSegmentsInBranch()
        {
            return numSegmentsInBranch;
        }

        //offset as percentage of segment length
        private float segmentOffset = 0.4f;

        public float SegmentOffset()
        {
            return segmentOffset;
        }

        public int NumFinalBranches()
        {
            return (int)this.GetValueFromVariation((float)numFinalBranches, (float)numFinalBranchesVariation);
        }

        private float lengthReductionFactor = 0.2f;

        public float LengthReductionFactor()
        {
            return lengthReductionFactor;
        }

        private float branchStartWidth = 0.1f;

        public float BranchStartWidth()
        {
            return branchStartWidth;
        }

        private float trunkStartWidth = 0.25f;
        public float TrunkStartWidth()
        {
            return trunkStartWidth;
        }

        private float trunkTaperPercent = 0.3f;
        public float TrunkTaperPercent()
        {
            return trunkTaperPercent;
        }

        private float minBranchWidth = 0.005f;

        public float MinBranchWidth()
        {
            return minBranchWidth;
        }

        private float taperPercent = 0.2f;

        public float MaxTrunkHeightPercent()
        {
            return 1.0f;
        }

        public float TaperPercent()
        {
            return taperPercent;
        }

        public int NumPointsOnLobe()
        {
            return numPointsOnLobe;
        }

        int numLeafPointsOnLobe = 50;
        public int NumLeafPointsOnLobe()
        {
            return numLeafPointsOnLobe;
        }

        public float Angle()
        {
            float a = GetValueFromVariation(angle, angleVariation);
            return a;
        }

        //clamp on [value - variation : value + variation]
        public float GetValueFromVariation(float value, float variation)
        {
            return value + ((float)game.rand.NextDouble() * (variation * 2)) - variation;
        }

        public void CreateLeaves(Game1 game, TreeBranch branch)
        {
            /*foreach (BoundingSphere sphere in branch.leafControlPoints)
            {
                branch.AddChild(new TreeBranch(game, branch, branch.end, sphere.Center, false));
            }

            game.parentForm.branchViewWindow.RefreshView();
            game.branchCreator.ResetBranchManager();*/

            #region fix with better leaf code
            //float dist = Math.Abs((branch.end - branch.start).Length());
            //float leafDim = 0.175f;
            ////create leaf branches
            //if (!branch.isTrunk) //create new leaf branch and add leaves to that
            //{
            //    for (float i = 0; i < dist; i += (leafDim * 1.5f))
            //    {
            //        Vector3 lateral = Vector3.Cross(Vector3.Up, branch.Dir());
            //        lateral.Normalize();
            //        Quad leaf = new Quad(game, branch.start + branch.Dir() * i + lateral * (float)game.rand.NextDouble() * leafDim * 1.25f, Vector3.Up, branch.Dir(), leafDim, leafDim);
            //        ((Game1)game).leafQuads.Add(leaf);
            //        leaf = new Quad(game, branch.start + branch.Dir() * i + Vector3.Up * (float)game.rand.NextDouble() * leafDim * 1.25f, Vector3.Up, branch.Dir(), leafDim, leafDim);
            //        ((Game1)game).leafQuads.Add(leaf);


            //        leaf = new Quad(game, branch.start + branch.Dir() * i - lateral * (float)game.rand.NextDouble() * leafDim * 1.25f, Vector3.Up, branch.Dir(), leafDim, leafDim);
            //        ((Game1)game).leafQuads.Add(leaf);
            //        leaf = new Quad(game, branch.start + branch.Dir() * i - Vector3.Up * (float)game.rand.NextDouble() * leafDim * 1.25f, Vector3.Up, branch.Dir(), leafDim, leafDim);
            //        ((Game1)game).leafQuads.Add(leaf);

            //        //create direction
            //        int spread = 15;
            //        float xSpread = branch.ClampSpread(game.rand.Next(-spread, spread), 5);
            //        float ySpread = branch.ClampSpread(game.rand.Next(-spread, spread), 5);
            //        float zSpread = branch.ClampSpread(game.rand.Next(-spread, spread), 5);

            //        Vector3 origin = branch.start + branch.Dir() * i;
            //        Vector3 rotation = new Vector3(xSpread, ySpread, zSpread);
            //        Vector3 twigEnd = branch.RotateAroundPoint(origin + branch.Dir() * i, origin, rotation);
            //        //dir = twigEnd - origin;

            //        for (float j = 0; j < dist; j += leafDim * 1.5f)
            //        {
            //            lateral = Vector3.Cross(Vector3.Up, branch.Dir());
            //            lateral.Normalize();

            //            ((Game1)game).leafQuads.Add(new Quad(game, branch.start + branch.Dir() * i + (float)game.rand.NextDouble() * lateral * leafDim * 1.25f, Vector3.Up, branch.Dir(), leafDim, leafDim));
            //            ((Game1)game).leafQuads.Add(new Quad(game, branch.start + branch.Dir() * i - (float)game.rand.NextDouble() * lateral * leafDim * 1.25f, Vector3.Up, branch.Dir(), leafDim, leafDim));

            //            ((Game1)game).leafQuads.Add(new Quad(game, branch.start + branch.Dir() * i + (float)game.rand.NextDouble() * Vector3.Up * leafDim * 1.25f, Vector3.Up, branch.Dir(), leafDim, leafDim));
            //            ((Game1)game).leafQuads.Add(new Quad(game, branch.start + branch.Dir() * i - (float)game.rand.NextDouble() * Vector3.Up * leafDim * 1.25f, Vector3.Up, branch.Dir(), leafDim, leafDim));
            //        }
            //    }
            //}
            #endregion
        }

        public Texture2D LeafTexture()
        {
            return game.cherryBlossom;
        }

        public List<LeafLobe> CreateBranchControlPoints(List<LeafLobe> leafLobes)
        {
            foreach (LeafLobe lobe in leafLobes)
            {
                lobe.controlPoints = new List<BoundingSphere>();
                for (int i = 0; i < NumPointsOnLobe(); i++)
                {
                    Vector3 point = game.GetRandomPointOnSphere(lobe.sphere, -3, -3);
                    lobe.controlPoints.Add(new BoundingSphere(point, 0.1f));
                }
            }

            return leafLobes;
        }

        public List<LeafLobe> CreateLeafControlPoints(List<LeafLobe> leafLobes)
        {
            foreach (LeafLobe lobe in leafLobes)
            {
                lobe.leafControlPoints = new List<BoundingSphere>();
                for (int i = 0; i < NumLeafPointsOnLobe(); i++)
                {
                    Vector3 point = game.GetRandomPointOnSphere(lobe.sphere, 1, 1);
                    lobe.leafControlPoints.Add(new BoundingSphere(point, 0.1f));
                }
            }

            return leafLobes;
        }

        public void ConnectBranchesToLeafLobes()
        {
            List<TreeBranch> terminatingBranches = game.branchCreator.treeHierarchy.GetTerminatingBranches(false);

            foreach (LeafLobe lobe in game.leafCreator.spheres)
            {
                foreach (BoundingSphere sphere in lobe.leafControlPoints)
                {
                    TreeBranch closestBranch = null;
                    float closestDist = float.MaxValue;

                    foreach (TreeBranch branch in terminatingBranches)
                    {
                        float dist = Vector3.Distance(branch.end, sphere.Center);

                        if (dist < closestDist)
                        {
                            closestDist = dist;
                            closestBranch = branch;
                        }
                    }

                    if (closestBranch != null)
                    {
                        TreeBranch connection = new TreeBranch(game, closestBranch, closestBranch.end, sphere.Center, BranchType.Twig);
                        //closestBranch.AddChild();

                        TreeBranch connectionArcParent = createArc(connection, lobe, 5);

                        AddLeaves(connectionArcParent);

                        closestBranch.AddChild(connectionArcParent);
                    }
                }
            }

            game.parentForm.branchViewWindow.RefreshView();
            game.branchCreator.ResetBranchManager();
        }

        private int numFlowersOnBranch = 3;
        private float flowerTwigLength = 0.2f;
        private float flowerTwigLengthVariation = 0.1f;

        private float GetFlowerTwigLength()
        {
            return GetValueFromVariation(flowerTwigLength, flowerTwigLengthVariation);
        }

        private float flowerTwigDirOffset = 0.0001f;

        private float leafDim = 0.175f;

        private void AddLeaves(TreeBranch start)
        {
            List<TreeBranch> children = game.branchCreator.treeHierarchy.GetAllChildren(start);
            children.Add(start);
            //Vector3 parentDir = start.parent.Dir();
            foreach (TreeBranch child in children)
            {
                Vector3 parentDir = Vector3.Zero;
                
                if (child.parent.branchType != BranchType.Twig)
                {
                    parentDir = child.children[0].Dir();
                }
                else
                {
                    parentDir = child.parent.Dir();
                }
                Vector3 lateral = Vector3.Cross(child.Dir(), parentDir);
                lateral.Normalize();

                for (int i = 0; i < numFlowersOnBranch; i++)
                {
                    Vector3 s = child.start + child.Dir() * (i + 1) * (child.Length() / (float)numFlowersOnBranch);
                    Vector3 e = s + lateral * GetFlowerTwigLength();
                    e += child.Dir();// *flowerTwigDirOffset;

                    Vector3 dir = e - s;
                    dir.Normalize();

                    e = s + dir * GetFlowerTwigLength();

                    TreeBranch leafTwig = new TreeBranch(game, child, s, e, BranchType.Twig);
                    child.AddChild(leafTwig);

                    Quad leaf = new Quad(game, leafTwig.end, leafTwig.Dir(), Vector3.Up, leafDim, leafDim);
                    leafTwig.tree.AddLeaf(leaf, LeafTexture());
                }

                for (int i = 0; i < numFlowersOnBranch; i++)
                {
                    Vector3 s = child.start + child.Dir() * ((float)i + 0.5f) * (child.Length() / (float)numFlowersOnBranch);
                    Vector3 e = s - lateral * GetFlowerTwigLength();
                    e += child.Dir();// *flowerTwigDirOffset;

                    Vector3 dir = e - s;
                    dir.Normalize();

                    e = s + dir * GetFlowerTwigLength();

                    TreeBranch leafTwig = new TreeBranch(game, child, s, e, BranchType.Twig);
                    child.AddChild(leafTwig);

                    Quad leaf = new Quad(game, leafTwig.end, leafTwig.Dir(), Vector3.Up, leafDim, leafDim);
                    leafTwig.tree.AddLeaf(leaf, LeafTexture());
                }
            }
        }

        private TreeBranch createArc(TreeBranch connection, LeafLobe lobe, int numSegments)
        {
            List<TreeBranch> branches = new List<TreeBranch>();

            float dist = connection.Length();
            float segmentLength = dist / (float)numSegments;
            
            for (int i = 0; i < numSegments; i++)
            {
                Vector3 start;
                TreeBranch parent = connection.parent;

                if (branches.Count == 0)
                {
                    start = connection.start;
                }
                else
                {
                    parent = branches[branches.Count - 1];
                    start = parent.end;
                }

                Vector3 end = connection.start + (connection.Dir() * (i + 1) * segmentLength);
                Vector3 dir = end - lobe.sphere.Center;
                dir.Normalize();

                end = lobe.sphere.Center + dir * lobe.sphere.Radius;

                TreeBranch segment = new TreeBranch(game, parent, start, end, BranchType.Twig);
                parent.AddChild(segment);
                branches.Add(segment);
            }

            return branches[0];
        }

        public float TwigStartWidth()
        {
            return 0.005f;
        }

        public int Tesselation()
        {
            return 5;
        }

    }
}
