﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace kinectApp
{
    public enum BranchType
    {
        Trunk,
        Branch,
        Twig
    };

    public class TreeBranch
    {
        public TreeBranch parent = null;
        public List<TreeBranch> children = new List<TreeBranch>();
        public Vector3 start;
        public Vector3 end;
        private static int staticId = 0;
        public int id;
        public float lerpAmount = 1; //amount to lerp on parent branch
        public LeafLobe leafLobe = null;

        public Boolean isInsideLobe = false;

        //public Boolean isTrunk = false;
        Game1 game;

        public float branchStartWidth;

        public List<BoundingSphere> controlPoints = new List<BoundingSphere>();
        public List<BoundingSphere> leafControlPoints = new List<BoundingSphere>();

        public BranchType branchType;

        public Tree tree;

        public TreeBranch(Game game, TreeBranch parent, Vector3 start, Vector3 end, BranchType branchType)
        {
            this.game = (Game1)game;
            //this.isTrunk = isTrunk;
            this.branchType = branchType;
            Initialize(parent, start, end, staticId);

            staticId++;
        }

        public Boolean addLeaves = false;

        /*public void SetBranchType(BranchType type)
        {
            this.branchType = type;
        }*/

        /// <summary>
        /// Create a TreeBranch with a random end point
        /// </summary>
        /// <param name="game"></param>
        /// <param name="parent"></param>
        /// <param name="start"></param>
        /// <param name="isTrunk"></param>
        public TreeBranch(Game game, TreeBranch parent, Vector3 start, float length, float angle, BranchType branchType)
        {
            this.game = (Game1)game;
            //this.isTrunk = isTrunk;
            this.branchType = branchType;

            //TODO: create end based on rules
            //this.game.TreeRules().Length();
            Vector3 parentDir = parent == null ? Vector3.Up : parent.Dir();
            parentDir.Normalize();
            Vector3 rotAxis = GetRotationAxis(parentDir);
            float rot = (float)this.game.rand.NextDouble() * 360.0f;

            RotateAroundAxis(/*this.game.TreeRules().Angle()*/ angle, start + parentDir * length, rotAxis);
            RotateAroundAxis(rot, this.end, parentDir);

            if (Vector3.Distance(this.start, this.end) > length)
            {
                Vector3 dir = this.end - start;
                dir.Normalize();
                this.end = start + dir * length;
            }

            Initialize(parent, start, end, staticId);

            staticId++;
        }

        public TreeBranch(Game game, TreeBranch parent, Vector3 start, float length, float angle, float rotAround, BranchType branchType)
        {
            this.game = (Game1)game;
            //this.isTrunk = isTrunk;

            this.branchType = branchType;

            //TODO: create end based on rules
            //this.game.TreeRules().Length();
            Vector3 parentDir = parent == null ? Vector3.Up : parent.Dir();
            parentDir.Normalize();
            Vector3 rotAxis = GetRotationAxis(parentDir);
            //float rot = (float)this.game.rand.NextDouble() * 360.0f;

            //rotAxis = GetRotatedVector(parent.end, this.end

            RotateAroundAxis(/*this.game.TreeRules().Angle()*/ angle, start + parentDir * length, rotAxis);

            //RotateAroundAxis(rotAround, this.end, parent.Dir());

            this.end = GetRotatedVector(parent.end, this.end, rotAround, parent.Dir());

            //Vector3 newRotAxis = Vector3.Cross(parent.Dir(), Vector3.Backward);

            //RotateAroundAxis(rotAround, this.end, parentDir);


            if (Vector3.Distance(this.start, this.end) > length)
            {
                Vector3 dir = this.end - start;
                dir.Normalize();
                this.end = start + dir * length;
            }

            //this.end.Y = height;

            Initialize(parent, start, end, staticId, false);

            staticId++;
        }

        #region rotation functions
        private void RotateAroundAxis(float amount, Vector3 end, Vector3 axis)
        {
            this.end = GetRotatedVector(start, end, amount, axis);
        }

        public Vector3 GetRotationAxis(Vector3? parentDir = null)
        {
            //if null, assume parent dir = Vector3.Up
            Vector3 rotAxis = (parentDir == null || parentDir == Vector3.Up) ? 
                Vector3.Backward : 
                Vector3.Cross(Vector3.Up, (Vector3)parentDir);

            return rotAxis;
        }

        public Vector3 GetRotatedVector(Vector3 origin, Vector3 endPoint, float rotation, Vector3 rotationAxis)
        {
            return Vector3.Transform(endPoint - origin,
                Quaternion.CreateFromAxisAngle(rotationAxis, MathHelper.ToRadians(rotation))) + origin;
        }
        #endregion

        public Vector3 Dir()
        {
            Vector3 dir = end - start;
            dir.Normalize();
            return dir;
        }

        public Boolean hasCap = false;
        public float capDiameter = 0;
        public void AddCap()
        {
            float taperPercent = 1;

            switch (branchType)
            {
                case BranchType.Branch:
                    taperPercent = game.TreeRules().TaperPercent();
                    break;
                case BranchType.Trunk:
                    taperPercent = game.TreeRules().TrunkTaperPercent();
                    break;
            }

            this.hasCap = true;
            this.capDiameter = this.branchStartWidth * game.TreeRules().branchSphereConnectorScaleFactor();
        }

        public float Length()
        {
            return Vector3.Distance(start, end);
        }

        /// <summary>
        /// get random position on Branch
        /// </summary>
        public Vector3 GetPositionOnBranch()
        {
            return GetPositionOnBranch((float)game.rand.NextDouble());
        }

        /// <summary>
        /// get position on branch x-percent from start
        /// </summary>
        public Vector3 GetPositionOnBranch(float percent)
        {
            percent = MathHelper.Clamp(percent, 0, 1);
            return Vector3.Lerp(this.start, this.end, percent);
        }

        public static void Reset()
        {
            staticId = 0;
        }

        public static void SetStaticId(int id)
        {
            staticId = id;
        }

        private TreeBranch(TreeBranch parent, Vector3 start, Vector3 end, int id)
        {
            Initialize(parent, start, end, id);
        }

        private void Initialize(TreeBranch parent, Vector3 start, Vector3 end, int id, Boolean addLeaves = true)
        {
            this.tree = ((Game1)game).GetActiveTree();
            this.addLeaves = addLeaves;
            this.parent = parent;
            this.start = start;
            this.end = end;
            this.id = id;

            if (parent != null && parent.branchType == branchType)
            {
                this.lerpAmount = 1 - Vector3.Distance(start, parent.end) / parent.Length();
                if (branchType == BranchType.Trunk) //isTrunk)
                {
                    branchStartWidth = parent.branchStartWidth - //parent.branchStartWidth * (parent.lerpAmount * game.TreeRules().taperPercent);
                        parent.branchStartWidth * game.TreeRules().TrunkTaperPercent();
                }
                else
                {
                    float width;

                    if (parent.branchType == BranchType.Trunk) //parent.isTrunk)
                    {
                        width = game.TreeRules().BranchStartWidth();
                    }
                    else
                    {
                        width = parent.branchStartWidth;
                    }
                    branchStartWidth = width - //parent.branchStartWidth * (parent.lerpAmount * game.TreeRules().taperPercent);
                        (width * game.TreeRules().TaperPercent());
                }
            }
            else
            {
                switch (branchType)
                {
                    case (BranchType.Trunk):
                        branchStartWidth = game.TreeRules().TrunkStartWidth();
                        break;

                    case (BranchType.Branch):
                        branchStartWidth = game.TreeRules().BranchStartWidth();
                        break;

                    case (BranchType.Twig):
                        branchStartWidth = game.TreeRules().TwigStartWidth();
                        break;
                }

                //if (isTrunk)
                //{
                //    branchStartWidth = game.TreeRules().TrunkStartWidth();
                //}
                //else
                //{
                //    branchStartWidth = game.TreeRules().BranchStartWidth();
                //}
            }

            if (branchStartWidth < game.TreeRules().MinBranchWidth())
            {
                branchStartWidth = game.TreeRules().MinBranchWidth();
            }

            //if (addLeaves)
            //{
            //    AddLeaves();
            //}
        }

        public void AddLeaves()
        {
            game.TreeRules().CreateLeaves(game, this);
            //#region fix with better leaf code
            //float dist = Math.Abs((end - start).Length());
            //float leafDim = 0.175f;
            ////create leaf branches
            //if (!isTrunk) //create new leaf branch and add leaves to that
            //{
            //    for (float i = 0; i < dist; i += (leafDim * 1.5f))
            //    {
            //        Vector3 lateral = Vector3.Cross(Vector3.Up, Dir());
            //        lateral.Normalize();
            //        Quad leaf = new Quad(game, start + Dir() * i + lateral * (float)game.rand.NextDouble() * leafDim * 1.25f, Vector3.Up, Dir(), leafDim, leafDim);
            //        ((Game1)game).leafQuads.Add(leaf);
            //        leaf = new Quad(game, start + Dir() * i + Vector3.Up * (float)game.rand.NextDouble() * leafDim * 1.25f, Vector3.Up, Dir(), leafDim, leafDim);
            //        ((Game1)game).leafQuads.Add(leaf);


            //        leaf = new Quad(game, start + Dir() * i - lateral * (float)game.rand.NextDouble() * leafDim * 1.25f, Vector3.Up, Dir(), leafDim, leafDim);
            //        ((Game1)game).leafQuads.Add(leaf);
            //        leaf = new Quad(game, start + Dir() * i - Vector3.Up * (float)game.rand.NextDouble() * leafDim * 1.25f, Vector3.Up, Dir(), leafDim, leafDim);
            //        ((Game1)game).leafQuads.Add(leaf);

            //        //create direction
            //        int spread = 15;
            //        float xSpread = ClampSpread(game.rand.Next(-spread, spread), 5);
            //        float ySpread = ClampSpread(game.rand.Next(-spread, spread), 5);
            //        float zSpread = ClampSpread(game.rand.Next(-spread, spread), 5);

            //        Vector3 origin = start + Dir() * i;
            //        Vector3 rotation = new Vector3(xSpread, ySpread, zSpread);
            //        Vector3 twigEnd = RotateAroundPoint(origin + Dir() * i, origin, rotation);
            //        //dir = twigEnd - origin;

            //        for (float j = 0; j < dist; j += leafDim * 1.5f)
            //        {
            //            lateral = Vector3.Cross(Vector3.Up, Dir());
            //            lateral.Normalize();

            //            ((Game1)game).leafQuads.Add(new Quad(game, start + Dir() * i + (float)game.rand.NextDouble() * lateral * leafDim * 1.25f, Vector3.Up, Dir(), leafDim, leafDim));
            //            ((Game1)game).leafQuads.Add(new Quad(game, start + Dir() * i - (float)game.rand.NextDouble() * lateral * leafDim * 1.25f, Vector3.Up, Dir(), leafDim, leafDim));

            //            ((Game1)game).leafQuads.Add(new Quad(game, start + Dir() * i + (float)game.rand.NextDouble() * Vector3.Up * leafDim * 1.25f, Vector3.Up, Dir(), leafDim, leafDim));
            //            ((Game1)game).leafQuads.Add(new Quad(game, start + Dir() * i - (float)game.rand.NextDouble() * Vector3.Up * leafDim * 1.25f, Vector3.Up, Dir(), leafDim, leafDim));
            //        }
            //    }
            //}
            //#endregion
        }

        public float ClampSpread(float spread, float min)
        {
            if (spread < min && spread > 0)
            {
                spread = min;
            }
            if (spread > -min && spread < 0)
            {
                spread = -min;
            }

            return spread;
        }

        public Vector3 RotateAroundPoint(Vector3 point, Vector3 originPoint, Vector3 rotation)//float radiansToRotate)
        {
            Vector3 diffVect = point - originPoint;

            Vector3 rotatedVect = Vector3.Transform(diffVect, Matrix.CreateRotationX(MathHelper.ToRadians(rotation.X)) *
                Matrix.CreateRotationY(MathHelper.ToRadians(rotation.Y)) *
                Matrix.CreateRotationZ(MathHelper.ToRadians(rotation.Z)));

            rotatedVect += originPoint;

            return rotatedVect;
        }

        CylinderPrimitive cyl;
        SpherePrimitive sph;
        public void CreateVolume()
        {
            //get branch mat
            Matrix final_orientation = Matrix.Identity;
            if (Dir() != Vector3.Up)
            {
                final_orientation = Matrix.CreateFromYawPitchRoll(0, MathHelper.ToRadians(-90), 0) * Matrix.CreateWorld(start + ((end - start) / 2), Dir(), Vector3.Up);
            }
            else
            {
                final_orientation *= Matrix.CreateTranslation(start + (end - start) / 2);
            }

            float taperPercent = 1;

            switch (branchType)
            {
                case (BranchType.Trunk):
                    taperPercent = game.TreeRules().TrunkTaperPercent();
                    break;

                case (BranchType.Branch):
                    taperPercent = game.TreeRules().TaperPercent();
                    break;

                case (BranchType.Twig):
                    taperPercent = game.TreeRules().TaperPercent();
                    break;
            }

            cyl = new CylinderPrimitive(tree, game, game.GraphicsDevice, (end - start).Length(), branchStartWidth, taperPercent, game.TreeRules().Tesselation(), final_orientation);

            if (hasCap)
            {
                AddCap();
                sph = new SpherePrimitive(game, game.GraphicsDevice, this.capDiameter, 10, Matrix.CreateTranslation(end));
            }
        }

        public void AddChild(TreeBranch branch)
        {
            children.Add(branch);
        }

        #region copy
        public TreeBranch Copy()
        {
            TreeBranch copiedBranch = new TreeBranch(parent, start, end, id);

            List<TreeBranch> childRoot = new List<TreeBranch>();
            GetDeepCopyOfAllChildren(children, ref childRoot);
            copiedBranch.children = childRoot;

            return copiedBranch;
        }

        private void GetDeepCopyOfAllChildren(List<TreeBranch> branchSection, ref List<TreeBranch> allChildren)
        {
            foreach (TreeBranch branch in branchSection)
            {
                TreeBranch newCopiedChild = new TreeBranch(branch.parent, branch.start, branch.end, branch.id);
                GetDeepCopyOfAllChildren(branch.children, newCopiedChild);
                allChildren.Add(newCopiedChild);
            }
        }

        private void GetDeepCopyOfAllChildren(List<TreeBranch> branchSection, TreeBranch parent)
        {
            foreach (TreeBranch branch in branchSection)
            {
                TreeBranch newCopiedChild = new TreeBranch(game, branch.parent, branch.start, branch.end, branch.branchType);
                parent.children.Add(newCopiedChild);
                GetDeepCopyOfAllChildren(branch.children, newCopiedChild);
            }
        }
        #endregion

        public void Draw()
        {
            ////get branch mat
            //Matrix final_orientation = Matrix.Identity;
            //if (Dir() != Vector3.Up)
            //{
            //    final_orientation = Matrix.CreateFromYawPitchRoll(0, MathHelper.ToRadians(-90), 0) * Matrix.CreateWorld(start + ((end - start) / 2), Dir(), Vector3.Up);
            //}
            //else
            //{
            //    final_orientation *= Matrix.CreateTranslation(start + (end - start) / 2);
            //}
            //if (cyl != null)
            //{
            //    cyl.Draw(final_orientation, game.camera.View, game.camera.Projection, Color.SaddleBrown);
            //}

            foreach (TreeBranch node in children)
            {
                node.Draw();
            }
        }
    }
}
