﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace kinectApp
{
    public class TreeHierarchy
    {
        Game1 game;
        private List<TreeBranch> branches = new List<TreeBranch>();
        public List<TreeBranch> ParentBranches()
        {
            return branches;
        }

        public void setBranches(List<TreeBranch> branches)
        {
            this.branches = branches;
        }


        TreeHierarchyForm treeWindow;

        public TreeHierarchy(Game game)
        {
            this.game = (Game1)game;
            treeWindow = this.game.parentForm.branchViewWindow;
        }

        public void resetBranches()
        {
            branches = new List<TreeBranch>();
            TreeBranch.Reset();
        }

        public List<TreeBranch> GetAllBranches()
        {
            List<TreeBranch> allBranches = new List<TreeBranch>();
            GetAllBranches(branches, ref allBranches);
            return allBranches;
        }

        public List<TreeBranch> GetTerminatingBranches(Boolean includeTrunks = true)
        {
            List<TreeBranch> terminatingBranches = new List<TreeBranch>();
            foreach (TreeBranch branch in GetAllBranches())
            {
                if (branch.children.Count == 0)
                {
                    if (!includeTrunks)
                    {
                        //if (!branch.isTrunk)
                        if (branch.branchType != BranchType.Trunk)
                        {
                            terminatingBranches.Add(branch);
                        }
                    }
                    else
                    {
                        terminatingBranches.Add(branch);
                    }
                    
                }
            }
            return terminatingBranches;
        }

        public List<TreeBranch> GetAllChildren(TreeBranch parent)
        {
            List<TreeBranch> allBranches = new List<TreeBranch>();
            GetAllBranches(parent.children, ref allBranches);
            return allBranches;
        }

        private void GetAllBranches(List<TreeBranch> branchSection, ref List<TreeBranch> allBranches)
        {
            foreach (TreeBranch branch in branchSection)
            {
                allBranches.Add(branch);
                GetAllBranches(branch.children, ref allBranches);
            }
        }

        public void AddBranch(TreeBranch branch)
        {
            branches.Add(branch);
        }

        public void RemoveBranch(int id)
        {
            RemoveBranch(branches, id);
        }

        private void RemoveBranch(List<TreeBranch> branchSection, int id)
        {
            foreach (TreeBranch branch in branchSection)
            {
                if (branch.id == id)
                {
                    RemoveAllInSection(branch.children);
                    Console.WriteLine("Removed Branch: " + branch.id);
                    branchSection.Remove(branch);
                    return;
                }

                RemoveBranch(branch.children, id);
            }
        }

        private void RemoveAllInSection(List<TreeBranch> branches)
        {
            foreach (TreeBranch branch in branches)
            {
                RemoveBranch(branch.children, branch.id);
            }
        }

        public TreeBranch GetBranchById(string id)
        {
            //id = branch_#
            int num = Convert.ToInt32(id.Split('_')[1]);
            return GetBranchById(num);
        }

        public TreeBranch GetBranchById(int id)
        {
            TreeBranch ret = null;

            foreach (TreeBranch branch in GetAllBranches())
            {
                if (branch.id == id)
                {
                    ret = branch;
                    break;
                }
            }

            return ret;
        }

        public void UpdateNodeHierarchy(string parentName, string movedChildName)
        {
            TreeBranch parent = GetBranchById(parentName);
            TreeBranch child = GetBranchById(movedChildName);

            TreeBranch copy = child.Copy();
            copy.parent = parent;
            RemoveBranch(child.id);

            parent.AddChild(copy);
        }

        public void Update()
        {
        }
    }
}
