﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace kinectApp
{
    public partial class TreeHierarchyForm : Form
    {
        Game1 game;

        public TreeHierarchyForm(Game1 game)
        {
            this.game = game;
            InitializeComponent();

            this.branchTreeView.NodeMouseClick += treeViewNodeMouseClick;
            this.branchTreeView.ItemDrag += treeViewItemDrag;
            this.branchTreeView.DragEnter += treeViewDragEnter;
            this.branchTreeView.DragDrop += treeViewDragDrop;

            this.branchTreeView.KeyUp += treeViewKeyPress;

            this.FormClosed += formClosed;

            RefreshView();
            setSelectedNode();
        }

        public void formClosed(Object sender, EventArgs e)
        {
            game.branchCreator.DeselectBranches();
        }

        public void treeViewKeyPress(Object sender, EventArgs e)
        {
            setSelectedNode();
        }

        public void setSelectedNode()
        {
            if (game.branchCreator != null)
            {
                if (branchTreeView.SelectedNode != null)
                {
                    game.branchCreator.SetSelectedNode(branchTreeView.SelectedNode);
                }
                else
                {
                    game.branchCreator.selectedBranch = null;
                }
            }
        }

        #region drag/drop
        TreeNode sourceNode;
        public void treeViewItemDrag(Object sender, ItemDragEventArgs e)
        {
            try
            {
                sourceNode = (TreeNode)e.Item;
                DoDragDrop(e.Item.ToString(), DragDropEffects.Move);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in ItemDrag: " + ex.Message);
            }
        }

        private void treeViewDragEnter(object sender, DragEventArgs e)
        {
            try
            {
                if (e.Data.GetDataPresent(DataFormats.Text))
                {
                    e.Effect = DragDropEffects.Move;
                }
                else
                {
                    e.Effect = DragDropEffects.None;
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in DragEnter: " + ex.Message);
            }
        }

        private void treeViewDragDrop(object sender, DragEventArgs e)
        {
            try
            {
                Point pos = branchTreeView.PointToClient(new Point(e.X, e.Y));
                TreeNode targetNode = branchTreeView.GetNodeAt(pos);

                if (targetNode == null || sourceNode == null)
                {
                    return;
                }

                if (targetNode != null && sourceNode != null)
                {
                    if (GetChildren(sourceNode).Contains(targetNode))
                    {
                        return;
                    }

                    sourceNode.Remove();

                    if (sourceNode.Index > targetNode.Index)
                        targetNode.Nodes.Insert(targetNode.Index, sourceNode); //nodeCopy);

                    else
                        targetNode.Nodes.Insert(targetNode.Index + 1, sourceNode); //nodeCopy);

                    branchTreeView.Invalidate();
                    //TODO: reset node hierarchy in Model
                    //TODO: actions affect entire hierarchy in model, not just root!
                    //      TreeBranch needs Children
                    //      TODO: GetTreeBranchByStringID (instead of by int id - parse in func)
                    game.branchCreator.treeHierarchy.UpdateNodeHierarchy(targetNode.Text, sourceNode.Text);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in DragDrop: " + ex.Message);
            }
        }

        //TODO: move these to some other container?
        public List<TreeNode> GetChildren(TreeNode parentNode)
        {
            List<TreeNode> nodes = new List<TreeNode>();
            GetAllChildren(parentNode, ref nodes);
            return nodes;
        }

        public void GetAllChildren(TreeNode parentNode, ref List<TreeNode> nodes)
        {
            foreach (TreeNode childNode in parentNode.Nodes)
            {
                nodes.Add(childNode);
                GetAllChildren(childNode, ref nodes);
            }
        }
        #endregion

        public void treeViewNodeMouseClick(Object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                TreeNode selectedNode = branchTreeView.GetNodeAt(e.X, e.Y);
                Console.WriteLine(selectedNode.Text);
                game.branchCreator.SetSelectedNode(selectedNode);
            }
        }

        public void RefreshView()
        {
            if (!branchTreeView.IsDisposed)
            {
                if (game.branchCreator != null && game.branchCreator.treeHierarchy != null)
                {
                    branchTreeView.BeginUpdate();
                    branchTreeView.Nodes.Clear();
                    foreach (TreeBranch branch in game.branchCreator.treeHierarchy.ParentBranches())
                    {
                        TreeNode node = new TreeNode("branch_" + branch.id);
                        branchTreeView.Nodes.Add(node);

                        if (branch.children.Count > 0)
                        {
                            AddToTreeView(node, branch.children);
                        }
                    }

                    //foreach (TreeBranch branch in game.treeHierarchy. GetAllBranches())
                    //{
                    //    branchTreeView.Nodes.Add("branch_" + branch.id);
                    //}
                }
                branchTreeView.ExpandAll();
                branchTreeView.EndUpdate();
            }
        }

        public void AddToTreeView(TreeNode parent, List<TreeBranch> branches)
        {
            foreach (TreeBranch branch in branches)
            {
                TreeNode node = new TreeNode("branch_" + branch.id);
                parent.Nodes.Add(node);

                if (branch.children.Count > 0)
                {
                    AddToTreeView(node, branch.children);
                }
            }
        }
    }
}
